@@ -1,6 +1,6 @@
 /*
  *  Copyright (c) 2004 Michael Thaler <michael.thaler@physik.tu-muenchen.de> filters
- *  Copyright (c) 2005-2007 Casper Boemann <cbr@boemann.dk>
+ *  Copyright (c) 2005-2007 C. Boemann <cbo@boemann.dk>
  *  Copyright (c) 2005, 2010 Boudewijn Rempt <boud@valdyas.org>
  *  Copyright (c) 2010 Marc Pegon <pe.marc@free.fr>
  *
@@ -35,8 +35,9 @@
 #include "kis_debug.h"
 #include "kis_selection.h"
 #include "kis_datamanager.h"
-
-#include "kis_iterators_pixel.h"
+#include "kis_global.h"
+#include "kis_iterator_ng.h"
+#include "kis_random_accessor_ng.h"
 #include "kis_filter_strategy.h"
 #include "kis_layer.h"
 #include "kis_painter.h"
@@ -81,156 +82,141 @@ QTransform KisTransformWorker::transform() const
     return TS.inverted() * S * TS * SC * R * T;
 }
 
-void KisTransformWorker::rotateNone(KisPaintDeviceSP src, KisPaintDeviceSP dst)
+QRect KisTransformWorker::rotateNone(KisPaintDeviceSP src, KisPaintDeviceSP dst,
+                                     QRect boundRect,
+                                     KoUpdaterPtr progressUpdater,
+                                     int &lastProgressReport,
+                                     int &progressTotalSteps,
+                                     int &progressStep)
 {
     qint32 pixelSize = src->pixelSize();
-    QRect r(m_boundRect);
+    QRect r(boundRect);
     KoColorSpace *cs = src->colorSpace();
     Q_UNUSED(cs);
 
-    KisHLineIteratorPixel hit = src->createHLineIterator(r.x(), r.top(), r.width());
-    KisHLineIterator vit = dst->createHLineIterator(r.x(), r.top(), r.width());
+    KisHLineIteratorSP hit = src->createHLineIteratorNG(r.x(), r.top(), r.width());
+    KisHLineIteratorSP vit = dst->createHLineIteratorNG(r.x(), r.top(), r.width());
+
     for (qint32 i = 0; i < r.height(); ++i) {
-        while (!hit.isDone()) {
-            memcpy(vit.rawData(), hit.rawData(), pixelSize);
+        do {
+            memcpy(vit->rawData(), hit->rawData(), pixelSize);
+        } while (hit->nextPixel() && vit->nextPixel());
+        hit->nextRow();
+        vit->nextRow();
 
-            ++hit;
-            ++vit;
-        }
-        hit.nextRow();
-        vit.nextRow();
-
-        //progress info
-        m_progressStep += r.width();
-        if (m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
-            m_lastProgressReport = (m_progressStep * 100) / m_progressTotalSteps;
-            if (!m_progressUpdater.isNull()) m_progressUpdater->setProgress(m_lastProgressReport);
-        }
-        if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
-            break;
+        if (!progressUpdater.isNull()) {
+            //progress info
+            progressStep += r.width();
+            if (lastProgressReport != (progressStep * 100) / progressTotalSteps) {
+                lastProgressReport = (progressStep * 100) / progressTotalSteps;
+                if (!progressUpdater.isNull()) progressUpdater->setProgress(lastProgressReport);
+            }
+            if (progressUpdater->interrupted()) {
+                break;
+            }
         }
     }
+    return r;
 }
 
-void KisTransformWorker::rotateRight90(KisPaintDeviceSP src, KisPaintDeviceSP dst)
+QRect rotateWithTf(int rotation, KisPaintDeviceSP src, KisPaintDeviceSP dst,
+                   QRect boundRect,
+                   KoUpdaterPtr progressUpdater,
+                   int &lastProgressReport,
+                   int &progressTotalSteps,
+                   int &progressStep)
 {
     qint32 pixelSize = src->pixelSize();
-    QRect r(m_boundRect);
+    QRect r(boundRect);
     KoColorSpace *cs = src->colorSpace();
     Q_UNUSED(cs);
 
-    for (qint32 y = r.bottom(); y >= r.top(); --y) {
-        KisHLineIteratorPixel hit = src->createHLineIterator(r.x(), y, r.width());
-        KisVLineIterator vit = dst->createVLineIterator(-y, r.x(), r.width());
+    KisRandomAccessorSP srcAcc = src->createRandomAccessorNG(0, 0);
+    KisRandomAccessorSP dstAcc = dst->createRandomAccessorNG(0, 0);
 
-        while (!hit.isDone()) {
-            memcpy(vit.rawData(), hit.rawData(), pixelSize);
+    QTransform tf;
+    tf = tf.rotate(rotation);
 
-            ++hit;
-            ++vit;
+    int ty = 0;
+    int tx = 0;
+
+    for (qint32 y = r.y(); y <= r.height() + r.y(); ++y) {
+        for (qint32 x = r.x(); x <= r.width() + r.x(); ++x) {
+            tf.map(x, y, &tx, &ty);
+            srcAcc->moveTo(x, y);
+            dstAcc->moveTo(tx, ty);
+            memcpy(dstAcc->rawData(), srcAcc->oldRawData(), pixelSize);
         }
 
-        //progress info
-        m_progressStep += r.width();
-        if (m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
-            m_lastProgressReport = (m_progressStep * 100) / m_progressTotalSteps;
-            if (m_progressUpdater) m_progressUpdater->setProgress(m_lastProgressReport);
-        }
-        if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
-            break;
+        if (!progressUpdater.isNull()) {
+            // Progress info
+            progressStep += r.width();
+            if (lastProgressReport != (progressStep * 100) / progressTotalSteps) {
+                lastProgressReport = (progressStep * 100) / progressTotalSteps;
+                if (progressUpdater) progressUpdater->setProgress(lastProgressReport);
+            }
+            if (progressUpdater->interrupted()) {
+                break;
+            }
         }
     }
 
-    m_boundRect = QRect(- r.top() - r.height(), r.x(), r.height(), r.width());
+    return r;
 }
 
-void KisTransformWorker::rotateLeft90(KisPaintDeviceSP src, KisPaintDeviceSP dst)
+QRect KisTransformWorker::rotateRight90(KisPaintDeviceSP src, KisPaintDeviceSP dst,
+                                        QRect boundRect,
+                                        KoUpdaterPtr progressUpdater,
+                                        int &lastProgressReport,
+                                        int &progressTotalSteps,
+                                        int &progressStep)
 {
-    qint32 pixelSize = src->pixelSize();
-    QRect r(m_boundRect);
-    KoColorSpace *cs = src->colorSpace();
-    Q_UNUSED(cs);
-
-    KisHLineIteratorPixel hit = src->createHLineIterator(r.x(), r.top(), r.width());
-
-    for (qint32 y = r.top(); y <= r.bottom(); ++y) {
-        // Read the horizontal line from back to front, write onto the vertical column
-        KisVLineIterator vit = dst->createVLineIterator(y, -r.x() - r.width(), r.width());
-
-        hit += r.width() - 1;
-        while (!vit.isDone()) {
-            memcpy(vit.rawData(), hit.rawData(), pixelSize);
-
-            --hit;
-            ++vit;
-        }
-        hit.nextRow();
-
-        // Progress info
-        m_progressStep += r.width();
-        if (m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
-            m_lastProgressReport = (m_progressStep * 100) / m_progressTotalSteps;
-            if (m_progressUpdater) m_progressUpdater->setProgress(m_lastProgressReport);
-        }
-        if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
-            break;
-        }
-    }
-
-    m_boundRect = QRect(r.top(), - r.x() - r.width(), r.height(), r.width());
+    QRect r = rotateWithTf(90, src, dst, boundRect, progressUpdater, lastProgressReport, progressTotalSteps, progressStep);
+    dst->move(dst->x() - 1, dst->y());
+    return QRect(- r.top() - r.height(), r.x(), r.height(), r.width());
 }
 
-void KisTransformWorker::rotate180(KisPaintDeviceSP src, KisPaintDeviceSP dst)
+QRect KisTransformWorker::rotateLeft90(KisPaintDeviceSP src, KisPaintDeviceSP dst,
+                                       QRect boundRect,
+                                       KoUpdaterPtr progressUpdater,
+                                       int &lastProgressReport,
+                                       int &progressTotalSteps,
+                                       int &progressStep)
 {
-    qint32 pixelSize = src->pixelSize();
-    QRect r(m_boundRect);
-    KoColorSpace *cs = src->colorSpace();
-    Q_UNUSED(cs);
+    QRect r = rotateWithTf(270, src, dst, boundRect, progressUpdater, lastProgressReport, progressTotalSteps, progressStep);
+    dst->move(dst->x(), dst->y() -1);
+    return QRect(r.top(), - r.x() - r.width(), r.height(), r.width());
+}
 
-    KisHLineIteratorPixel srcIt = src->createHLineIterator(r.x(), r.top(), r.width());
-
-    for (qint32 y = r.top(); y <= r.bottom(); ++y) {
-        KisHLineIterator dstIt = dst->createHLineIterator(-r.x() - r.width(), -y, r.width());
-
-        srcIt += r.width() - 1;
-        while (!dstIt.isDone()) {
-            memcpy(dstIt.rawData(), srcIt.rawData(), pixelSize);
-            --srcIt;
-            ++dstIt;
-        }
-        srcIt.nextRow();
-
-        //progress info
-        m_progressStep += r.width();
-        if (m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
-            m_lastProgressReport = (m_progressStep * 100) / m_progressTotalSteps;
-            if (m_progressUpdater) m_progressUpdater->setProgress(m_lastProgressReport);
-        }
-        if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
-            break;
-        }
-    }
-
-    m_boundRect = QRect(- r.x() - r.width(), - r.top() - r.height(), r.width(), r.height());
+QRect KisTransformWorker::rotate180(KisPaintDeviceSP src, KisPaintDeviceSP dst,
+                                    QRect boundRect,
+                                    KoUpdaterPtr progressUpdater,
+                                    int &lastProgressReport,
+                                    int &progressTotalSteps,
+                                    int &progressStep)
+{
+    QRect r = rotateWithTf(180, src, dst, boundRect, progressUpdater, lastProgressReport, progressTotalSteps, progressStep);
+    dst->move(dst->x() - 1, dst->y() -1);
+    return QRect(- r.x() - r.width(), - r.top() - r.height(), r.width(), r.height());
 }
 
 template <class iter> iter createIterator(KisPaintDevice *dev, qint32 start, qint32 lineNum, qint32 len);
 
-template <> KisHLineIteratorPixel createIterator <KisHLineIteratorPixel>
+template <> KisHLineIteratorSP createIterator <KisHLineIteratorSP>
 (KisPaintDevice *dev, qint32 start, qint32 lineNum, qint32 len)
 {
-    return dev->createHLineIterator(start, lineNum, len);
+    return dev->createHLineIteratorNG(start, lineNum, len);
 }
 
-template <> KisVLineIteratorPixel createIterator <KisVLineIteratorPixel>
+template <> KisVLineIteratorSP createIterator <KisVLineIteratorSP>
 (KisPaintDevice *dev, qint32 start, qint32 lineNum, qint32 len)
 {
-    return dev->createVLineIterator(lineNum, start, len);
+    return dev->createVLineIteratorNG(lineNum, start, len);
 }
 
 template <class iter> void calcDimensions(QRect rc, qint32 &srcStart, qint32 &srcLen, qint32 &firstLine, qint32 &numLines);
 
-template <> void calcDimensions <KisHLineIteratorPixel>
+template <> void calcDimensions <KisHLineIteratorSP>
 (QRect rc, qint32 &srcStart, qint32 &srcLen, qint32 &firstLine, qint32 &numLines)
 {
     srcStart = rc.x();
@@ -239,7 +225,7 @@ template <> void calcDimensions <KisHLineIteratorPixel>
     numLines = rc.height();
 }
 
-template <> void calcDimensions <KisVLineIteratorPixel>
+template <> void calcDimensions <KisVLineIteratorSP>
 (QRect rc, qint32 &srcStart, qint32 &srcLen, qint32 &firstLine, qint32 &numLines)
 {
     firstLine = rc.x();
@@ -250,7 +236,7 @@ template <> void calcDimensions <KisVLineIteratorPixel>
 
 template <class iter> void updateBounds(QRect &boundRect, double floatscale, double shear, qint32 dx);
 
-template <> void updateBounds <KisHLineIteratorPixel>
+template <> void updateBounds <KisHLineIteratorSP>
 (QRect &boundRect, double floatscale, double shear, qint32 dx)
 {
     // Do not use QRect::right()! Never! See Qt doc for info...
@@ -271,7 +257,7 @@ template <> void updateBounds <KisHLineIteratorPixel>
     boundRect = newRect.toAlignedRect();
 }
 
-template <> void updateBounds <KisVLineIteratorPixel>
+template <> void updateBounds <KisVLineIteratorSP>
 (QRect &boundRect, double floatscale, double shear, qint32 dx)
 {
     // Do not use QRect::bottom() as well! Never! See Qt doc for info...
@@ -313,7 +299,6 @@ void KisTransformWorker::transformPass(KisPaintDevice *src, KisPaintDevice *dst,
     KoColorSpace * cs = src->colorSpace();
     KoMixColorsOp * mixOp = cs->mixColorsOp();
     qint32 scale;
-    qint32 scaleDenom;
     qint32 shearFracOffset;
 
 
@@ -325,9 +310,7 @@ void KisTransformWorker::transformPass(KisPaintDevice *src, KisPaintDevice *dst,
         return;
     }
 
-    scaleDenom = srcLen;
-
-    if (scaleDenom == 0) {
+    if (srcLen == 0) {
         updateBounds <T>(m_boundRect, floatscale, shear, dx);
         return;
     }
@@ -337,12 +320,12 @@ void KisTransformWorker::transformPass(KisPaintDevice *src, KisPaintDevice *dst,
     qint32 invfscale = 256;
 
     // handle magnification/minification
-    if (abs(scale) < scaleDenom) {
-        support *= scaleDenom;
+    if (abs(scale) < srcLen) {
+        support *= srcLen;
         support /= scale;
 
         invfscale *= scale;
-        invfscale /= scaleDenom;
+        invfscale /= srcLen;
         if (scale < 0) { // handle mirroring
             support = -support;
             invfscale = -invfscale;
@@ -377,10 +360,8 @@ void KisTransformWorker::transformPass(KisPaintDevice *src, KisPaintDevice *dst,
         qint32 sum = 0;
         for (int num = 0; num < span; ++num) {
             qint32 tmpw = filterStrategy->intValueAt(t) * invfscale;
-
             tmpw >>= 8;
             filterWeights[center].weight[num] = tmpw;
-
             t += dt;
             sum += tmpw;
         }
@@ -409,10 +390,11 @@ void KisTransformWorker::transformPass(KisPaintDevice *src, KisPaintDevice *dst,
 
     // Now time to do the actual scaling and shearing
     for (lineNum = firstLine; lineNum < firstLine + numLines; lineNum++) {
-        if (scale < 0)
-            dstStart = srcStart * scale / scaleDenom - dstLen + dx;
-        else
-            dstStart = (srcStart) * scale / scaleDenom + dx;
+        if (scale < 0) {
+            dstStart = srcStart * scale / srcLen - dstLen + dx;
+        } else {
+            dstStart = (srcStart) * scale / srcLen + dx;
+        }
 
         shearFracOffset = -int(256 * (lineNum * shear - floor(lineNum * shear)));
         dstStart += int(floor(lineNum * shear));
@@ -421,57 +403,73 @@ void KisTransformWorker::transformPass(KisPaintDevice *src, KisPaintDevice *dst,
         T srcIt = createIterator <T>(src, srcStart, lineNum, srcLen);
         quint8 *data;
         qint32 i = 0;
-        data = srcIt.rawData(); // take the first pixel as source - in effect duplicating the pixel
+        data = srcIt->rawData(); // take the first pixel as source - in effect duplicating the pixel
         while (i < extraLen) {
             memcpy(&tmpLine[i*pixelSize], data, pixelSize);
             i++;
         }
 
+
         while (i < srcLen + extraLen) {
-            data = srcIt.rawData();
+            data = srcIt->rawData();
             memcpy(&tmpLine[i*pixelSize], data, pixelSize);
             memcpy(data, src->defaultPixel(), pixelSize);
-            if (i < srcLen + extraLen - 1) // duplicate pixels along edge
-                ++srcIt;
+            if (i < srcLen + extraLen - 1) { // duplicate pixels along edge
+                srcIt->nextPixel();
+            }
             i++;
         }
 
+
+
         data = &tmpLine[(i-1)*pixelSize]; // take the last pixel as source - in effect duplicating the pixel
         while (i < srcLen + 2*extraLen) {
             memcpy(&tmpLine[i*pixelSize], data, pixelSize);
             i++;
         }
+        // line is done
 
         T dstIt = createIterator <T>(dst, dstStart, lineNum, dstLen);
 
         i = 0;
-        while (!dstIt.isDone()) {
-            if (scaleDenom < 2500)
-                center = ((i << 8) * scaleDenom) / scale;
-            else {
-                if (scaleDenom < 46000) // real limit is actually 46340 pixels
-                    center = ((i * scaleDenom) / scale) << 8;
-                else
-                    center = ((i << 8) / scale * scaleDenom) / scale; // XXX fails for sizes over 2^23 pixels src width
+
+        do {
+            // NOTE: 2500 was selected by Boemann as limit to avoid overflow
+            // but the overflow happens when dstLen is bigger then 3356, thus
+            // new condition to avoid the overflow.
+            // How did I caluculated the magic number? (qint32_max / 256) / (2500-1) = ~3356
+            if ((srcLen < 2500) && (dstLen < 3300)) {
+                center = ((i << 8) * srcLen) / scale;
+            } else {
+                if (srcLen < 46000) { // real limit is actually 46340 pixels
+                    center = ((i * srcLen) / scale) << 8;
+                } else {
+                    center = ((i << 8) / scale * srcLen) / scale; // XXX fails for sizes over 2^23 pixels src width
+                }
             }
 
-            if (scale < 0)
+            if (scale < 0) {
                 center += srcLen << 8;
+            }
 
             // Since the above gives us the position at the left of the pixels we need to advance by one half dst pixel
-            center += 128 * scaleDenom / scale;//xxx doesn't work for scale<0;
+            center += 128 * srcLen / scale;//xxx doesn't work for scale<0;
             center += (extraLen << 8) + shearFracOffset;
 
             // find contributing pixels
             begin = (center - support) >> 8; // find first pixel to sample
             end = (center + support) >> 8; // find last pixel to sample
 
+            // catch fixed-arithmethics overflows indirectly (variable center can overflow)
+            Q_ASSERT(begin >= 0);
+            Q_ASSERT(end >= 0);
+
             int num = 0;
             for (int srcpos = begin; srcpos <= end; ++srcpos) {
                 colors[num] = &tmpLine[srcpos*pixelSize];
                 num++;
             }
-            data = dstIt.rawData();
+            data = dstIt->rawData();
             mixOp->mixColors(colors, filterWeights[center&255].weight, filterWeights[center&255].numWeights, data);
 
             /*
@@ -479,9 +477,8 @@ void KisTransformWorker::transformPass(KisPaintDevice *src, KisPaintDevice *dst,
                         if (fixBorderAlpha && (i == 0 || i == dstLen - 1))
                             cs->setAlpha(data, cs->alpha(&tmpLine[(center>>8)*pixelSize]), 1);
             */
-            ++dstIt;
             i++;
-        }
+        } while (dstIt->nextPixel());
 
         //progress info
         m_progressStep += dstLen;
@@ -508,14 +505,14 @@ bool KisTransformWorker::run()
     Q_ASSERT_X(m_yscale != 0, "KisTransformer::run() validation step", "yscale == 0");
     // Fallback safety line in case Krita is compiled without ASSERTS
     if (m_xscale == 0 || m_yscale == 0) return false;
-    
+
     // Progress info
-    m_progressTotalSteps = 0;
+    m_progressTotalSteps = 1;
     m_progressStep = 0;
 
     KoColor defaultPixel(m_dev->defaultPixel(), m_dev->colorSpace());
     if (defaultPixel.opacityU8() != OPACITY_TRANSPARENT_U8)
-        m_boundRect = m_dev->dataManager()->extent();
+        m_boundRect = m_dev->extent();
     else
         m_boundRect = m_dev->exactBounds();
 
@@ -525,8 +522,10 @@ bool KisTransformWorker::run()
         return true;
     }
 
-    KisPaintDeviceSP tmpdev1 = KisPaintDeviceSP(new KisPaintDevice(m_dev->colorSpace()));
+
+    KisPaintDeviceSP tmpdev1 = new KisPaintDevice(m_dev->colorSpace());
     KisPaintDeviceSP srcdev = m_dev;
+    tmpdev1->setDefaultPixel(srcdev->defaultPixel());
 
     double xscale = m_xscale;
     double yscale = m_yscale;
@@ -536,17 +535,16 @@ bool KisTransformWorker::run()
     qint32 xtranslate = m_xtranslate;
     qint32 ytranslate = m_ytranslate;
 
-    m_progressTotalSteps = 0;
-
     // Apply shear X and Y
     if (xshear != 0 || yshear != 0) {
         m_progressTotalSteps += (yscale * m_boundRect.height() * (m_boundRect.width() + m_xshear * m_boundRect.height()));
         m_progressTotalSteps += (xscale * m_boundRect.width() * (m_boundRect.height() + m_yshear * m_boundRect.width()));
 
+
         int dx = - qRound(m_yshearOrigin * yscale * m_xshear);
         int dy = - qRound(m_xshearOrigin * xscale * m_yshear);
-        transformPass <KisHLineIteratorPixel>(srcdev.data(), srcdev.data(), xscale, yscale *  m_xshear, dx, m_filter, m_fixBorderAlpha);
-        transformPass <KisVLineIteratorPixel>(srcdev.data(), srcdev.data(), yscale, m_yshear, dy, m_filter, m_fixBorderAlpha);
+        transformPass <KisHLineIteratorSP>(srcdev.data(), srcdev.data(), xscale, yscale *  m_xshear, dx, m_filter, m_fixBorderAlpha);
+        transformPass <KisVLineIteratorSP>(srcdev.data(), srcdev.data(), yscale, m_yshear, dy, m_filter, m_fixBorderAlpha);
         yscale = 1.;
         xscale = 1.;
     }
@@ -599,17 +597,17 @@ bool KisTransformWorker::run()
     case 0:
         break;
     case 1:
-        rotateRight90(srcdev, tmpdev1);
+        m_boundRect = rotateRight90(srcdev, tmpdev1, m_boundRect, m_progressUpdater, m_lastProgressReport, m_progressTotalSteps, m_progressStep);
         srcdev->clear();
         srcdev = tmpdev1;
         break;
     case 2:
-        rotate180(srcdev, tmpdev1);
+        m_boundRect = rotate180(srcdev, tmpdev1, m_boundRect, m_progressUpdater, m_lastProgressReport, m_progressTotalSteps, m_progressStep);
         srcdev->clear();
         srcdev = tmpdev1;
         break;
     case 3:
-        rotateLeft90(srcdev, tmpdev1);
+        m_boundRect = rotateLeft90(srcdev, tmpdev1, m_boundRect, m_progressUpdater, m_lastProgressReport, m_progressTotalSteps, m_progressStep);
         srcdev->clear();
         srcdev = tmpdev1;
         break;
@@ -618,15 +616,15 @@ bool KisTransformWorker::run()
     //// Handle simple move case possibly with rotation of 90,180,270
     if (rotation == 0.0 && xscale == 1.0 && yscale == 1.0) {
 
-        updateBounds <KisHLineIteratorPixel>(m_boundRect, 1.0, 0, xtranslate);
-        updateBounds <KisVLineIteratorPixel>(m_boundRect, 1.0, 0, ytranslate);
+        updateBounds <KisHLineIteratorSP>(m_boundRect, 1.0, 0, xtranslate);
+        updateBounds <KisVLineIteratorSP>(m_boundRect, 1.0, 0, ytranslate);
 
         if (rotQuadrant == 0) {
             // When we didn't move the m_dev to a temp device we can simply just move its coords
             srcdev->move(srcdev->x() + xtranslate, srcdev->y() + ytranslate);
         } else {
             srcdev->move(srcdev->x() + xtranslate, srcdev->y() + ytranslate);
-            rotateNone(srcdev, m_dev); //copy it back
+            m_boundRect = rotateNone(srcdev, m_dev, m_boundRect, m_progressUpdater, m_lastProgressReport, m_progressTotalSteps, m_progressStep); //copy it back
         }
         //progress info
         if (!m_progressUpdater.isNull()) m_progressUpdater->setProgress(100);
@@ -639,7 +637,7 @@ bool KisTransformWorker::run()
     }
 
     // First pass
-    transformPass <KisHLineIteratorPixel>(srcdev.data(), srcdev.data(), xscale, yscale*xshear, 0, m_filter, m_fixBorderAlpha);
+    transformPass <KisHLineIteratorSP>(srcdev.data(), srcdev.data(), xscale, yscale*xshear, 0, m_filter, m_fixBorderAlpha);
 
     if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
         m_progressUpdater->setProgress(100);
@@ -647,7 +645,7 @@ bool KisTransformWorker::run()
     }
 
     // Now do the second pass
-    transformPass <KisVLineIteratorPixel>(srcdev.data(), srcdev.data(), yscale, yshear, ytranslate, m_filter, m_fixBorderAlpha);
+    transformPass <KisVLineIteratorSP>(srcdev.data(), srcdev.data(), yscale, yshear, ytranslate, m_filter, m_fixBorderAlpha);
 
     if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
         m_progressUpdater->setProgress(100);
@@ -655,14 +653,14 @@ bool KisTransformWorker::run()
     }
 
     if (xshear != 0.0) {
-        transformPass <KisHLineIteratorPixel>(srcdev.data(), m_dev.data(), 1.0, xshear, xtranslate, m_filter, m_fixBorderAlpha);
+        transformPass <KisHLineIteratorSP>(srcdev.data(), m_dev.data(), 1.0, xshear, xtranslate, m_filter, m_fixBorderAlpha);
     } else {
         // No need to filter again when we are only scaling
         srcdev->move(srcdev->x() + xtranslate, srcdev->y());
-        updateBounds <KisHLineIteratorPixel>(m_boundRect, 1.0, 0, xtranslate);
+        updateBounds <KisHLineIteratorSP>(m_boundRect, 1.0, 0, xtranslate);
 
         if (rotQuadrant != 0)  // no need to copy back if we have not copied the device in the first place
-            rotateNone(srcdev, m_dev);
+            m_boundRect = rotateNone(srcdev, m_dev, m_boundRect, m_progressUpdater, m_lastProgressReport, m_progressTotalSteps, m_progressStep);
     }
 
     //CBRm_dev->setDirty();
@@ -682,6 +680,7 @@ QRect KisTransformWorker::mirrorX(KisPaintDeviceSP dev, const KisSelection* sele
     KisPaintDeviceSP dst = new KisPaintDevice(dev->colorSpace());
 
     QRect r;
+
     if (selection) {
         r = selection->selectedExactRect();
     } else {
@@ -692,24 +691,38 @@ QRect KisTransformWorker::mirrorX(KisPaintDeviceSP dev, const KisSelection* sele
             r = dev->exactBounds();
     }
     {
-        KisHLineConstIteratorPixel srcIt = dev->createHLineConstIterator(r.x(), r.top(), r.width(), selection);
-        KisHLineIteratorPixel dstIt = dst->createHLineIterator(r.x(), r.top(), r.width());
+        quint8 *dstPixels = new quint8[r.width() * pixelSize];
+
+        KisHLineConstIteratorSP srcIt = dev->createHLineConstIteratorNG(r.x(), r.top(), r.width());
+
+        KisHLineConstIteratorSP selIt;
+        if (selection) {
+            selIt = selection->projection()->createHLineConstIteratorNG(r.x(), r.top(), r.width());
+        }
 
         for (qint32 y = r.top(); y <= r.bottom(); ++y) {
 
-            dstIt += r.width() - 1;
+            quint8 *dstIt = dstPixels + (r.width() * pixelSize) - pixelSize;
 
-            while (!srcIt.isDone()) {
-                if (srcIt.isSelected()) {
-                    memcpy(dstIt.rawData(), srcIt.rawData(), pixelSize);
+            do {
+                if (selIt) {
+                    if (*selIt->oldRawData() > SELECTION_THRESHOLD) {
+                        memcpy(dstIt, srcIt->oldRawData(), pixelSize);
+                        selIt->nextPixel();
+                    }
                 }
-                ++srcIt;
-                --dstIt;
+                else {
+                    memcpy(dstIt, srcIt->oldRawData(), pixelSize);
+                }
+                dstIt -= pixelSize;
 
-            }
-            srcIt.nextRow();
-            dstIt.nextRow();
+            } while (srcIt->nextPixel());
+
+            dst->writeBytes(dstPixels, QRect(r.left(), y, r.width(), 1));
+            srcIt->nextRow();
         }
+
+        delete[] dstPixels;
     }
     KisPainter gc(dev);
 
@@ -745,16 +758,23 @@ QRect KisTransformWorker::mirrorY(KisPaintDeviceSP dev, const KisSelection* sele
         qint32 y1, y2;
         for (y1 = r.top(), y2 = r.bottom(); y1 <= r.bottom(); ++y1, --y2) {
 
-            KisHLineConstIteratorPixel itTop = dev->createHLineConstIterator(r.x(), y1, r.width(), selection);
-            KisHLineIteratorPixel itBottom = dst->createHLineIterator(r.x(), y2, r.width());
-
-            while (!itTop.isDone() && !itBottom.isDone()) {
-                if (itTop.isSelected()) {
-                    memcpy(itBottom.rawData(), itTop.rawData(), pixelSize);
-                }
-                ++itBottom;
-                ++itTop;
+            KisHLineConstIteratorSP itTop = dev->createHLineConstIteratorNG(r.x(), y1, r.width());
+            KisHLineIteratorSP itBottom = dst->createHLineIteratorNG(r.x(), y2, r.width());
+            KisHLineConstIteratorSP selIt;
+            if (selection) {
+                selIt = selection->projection()->createHLineConstIteratorNG(r.x(), r.top(), r.width());
             }
+            do {
+                if (selIt) {
+                    if (*selIt->oldRawData() > SELECTION_THRESHOLD) {
+                        memcpy(itBottom->rawData(), itTop->oldRawData(), pixelSize);
+                        selIt->nextPixel();
+                    }
+                }
+                else {
+                    memcpy(itBottom->rawData(), itTop->oldRawData(), pixelSize);
+                }
+            } while (itTop->nextPixel() && itBottom->nextPixel());
         }
     }
     KisPainter gc(dev);
@@ -769,5 +789,6 @@ QRect KisTransformWorker::mirrorY(KisPaintDeviceSP dev, const KisSelection* sele
     gc.bitBlt(r.topLeft(), dst, r);
 
     return r;
+
 }
 
