@@ -73,6 +73,9 @@
 #include "kis_node_model.h"
 #include "canvas/kis_canvas2.h"
 #include "kis_doc2.h"
+#include "kis_dummies_facade_base.h"
+#include <KoShapeBasedDocumentBase.h>
+
 
 #include "ui_wdglayerbox.h"
 
@@ -90,13 +93,13 @@ KisLayerBox::KisLayerBox()
 
     m_wdgLayerBox->setupUi(mainWidget);
 
-    setMinimumSize(mainWidget->minimumSizeHint());
-
-    m_wdgLayerBox->listLayers->setDragDropMode(QAbstractItemView::InternalMove);
+    m_wdgLayerBox->listLayers->setDefaultDropAction(Qt::MoveAction);
     m_wdgLayerBox->listLayers->setVerticalScrollMode(QAbstractItemView::ScrollPerItem);
 
     connect(m_wdgLayerBox->listLayers, SIGNAL(contextMenuRequested(const QPoint&, const QModelIndex&)),
             this, SLOT(slotContextMenuRequested(const QPoint&, const QModelIndex&)));
+    connect(m_wdgLayerBox->listLayers, SIGNAL(collapsed(const QModelIndex&)), SLOT(slotCollapsed(const QModelIndex &)));
+    connect(m_wdgLayerBox->listLayers, SIGNAL(expanded(const QModelIndex&)), SLOT(slotExpanded(const QModelIndex &)));
 
     m_viewModeMenu = new KMenu(this);
     QActionGroup *group = new QActionGroup(this);
@@ -173,7 +176,7 @@ KisLayerBox::KisLayerBox()
     m_newCloneLayerAction = new KAction(KIcon("edit-copy"), i18n("&Clone Layer"), this);
     connect(m_newCloneLayerAction, SIGNAL(triggered(bool)), this, SLOT(slotNewCloneLayer()));
 
-    m_newShapeLayerAction = new KAction(KIcon("bookmark-new"), i18n("&Shape Layer"), this);
+    m_newShapeLayerAction = new KAction(KIcon("bookmark-new"), i18n("&Vector Layer"), this);
     connect(m_newShapeLayerAction, SIGNAL(triggered(bool)), this, SLOT(slotNewShapeLayer()));
 
     m_newAdjustmentLayerAction = new KAction(KIcon("view-filter"), i18n("&Filter Layer..."), this);
@@ -213,12 +216,16 @@ KisLayerBox::KisLayerBox()
     
     m_nodeModel = new KisNodeModel(this);
 
-    // connect model updateUI() to enable/disable controls
-    // connect(m_nodeModel, SIGNAL(nodeActivated(KisNodeSP)), SLOT(updateUI()));      NOTE: commented for temporary bug fix
-    connect(m_nodeModel, SIGNAL(nodeActivated(KisNodeSP)), SLOT(setCurrentNode(KisNodeSP)));  // NOTE: temporary bug fix - Pentalis
+    /**
+     * Connect model updateUI() to enable/disable controls.
+     * Note: nodeActivated() is connected seperately in setImage(), because
+     *       it needs particular order of calls: first the connection to the
+     *       node manager should be called, then updateUI()
+     */
     connect(m_nodeModel, SIGNAL(rowsInserted(const QModelIndex&, int, int)), SLOT(updateUI()));
     connect(m_nodeModel, SIGNAL(rowsRemoved(const QModelIndex&, int, int)), SLOT(updateUI()));
     connect(m_nodeModel, SIGNAL(rowsMoved(const QModelIndex&, int, int, const QModelIndex&, int)), SLOT(updateUI()));
+    connect(m_nodeModel, SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)), SLOT(updateUI()));
     connect(m_nodeModel, SIGNAL(modelReset()), SLOT(updateUI()));
 
     m_wdgLayerBox->listLayers->setModel(m_nodeModel);
@@ -229,76 +236,96 @@ KisLayerBox::~KisLayerBox()
     delete m_wdgLayerBox;
 }
 
+
+void expandNodesRecursively(KisNodeSP root, QPointer<KisNodeModel> nodeModel, KoDocumentSectionView *sectionView)
+{
+    if (!root) return;
+    if (nodeModel.isNull()) return;
+    if (!sectionView) return;
+
+    sectionView->blockSignals(true);
+
+    KisNodeSP node = root->firstChild();
+    while (node) {
+        QModelIndex idx = nodeModel->indexFromNode(node);
+        if (idx.isValid()) {
+            if (node->collapsed()) {
+                sectionView->collapse(idx);
+            }
+        }
+        if (node->childCount() > 0) {
+            expandNodesRecursively(node, nodeModel, sectionView);
+        }
+        node = node->nextSibling();
+    }
+    sectionView->blockSignals(false);
+}
+
 void KisLayerBox::setCanvas(KoCanvasBase *canvas)
 {
+
     if (m_canvas) {
-       m_canvas->disconnectCanvasObserver(this);
+        m_canvas->disconnectCanvasObserver(this);
+        m_nodeModel->setDummiesFacade(0, 0);
+
+        disconnect(m_image, 0, this, 0);
+        disconnect(m_nodeManager, 0, this, 0);
+        disconnect(m_nodeModel, 0, m_nodeManager, 0);
+        disconnect(m_nodeModel, SIGNAL(nodeActivated(KisNodeSP)), this, SLOT(updateUI()));
     }
 
     m_canvas = dynamic_cast<KisCanvas2*>(canvas);
+
     if (m_canvas) {
-        connect(m_canvas, SIGNAL(imageChanged(KisImageWSP)), SLOT(setImage(KisImageWSP)));
-        setImage(m_canvas->view()->image());
-    }
-}    
+        m_image = m_canvas->view()->image();
 
-
-void KisLayerBox::unsetCanvas()
-{
-    m_canvas = 0;
-}
-
-void KisLayerBox::setImage(KisImageWSP image)
-{
-    if(m_image && m_canvas && m_canvas->view())
-    {
-        disconnect(m_image, SIGNAL(sigAboutToBeDeleted()), this, SLOT(notifyImageDeleted()));
-        disconnect(m_nodeManager, SIGNAL(sigUiNeedChangeActiveNode(KisNodeSP)), this, SLOT(setCurrentNode(KisNodeSP)));
-        disconnect(m_nodeModel, SIGNAL(nodeActivated(KisNodeSP)), m_nodeManager, SLOT(slotUiActivatedNode(KisNodeSP)));
-
-        disconnect(m_nodeModel, SIGNAL(requestAddNode(KisNodeSP, KisNodeSP)), m_nodeManager, SLOT(addNode(KisNodeSP, KisNodeSP)));
-        disconnect(m_nodeModel, SIGNAL(requestAddNode(KisNodeSP, KisNodeSP, int)), m_nodeManager, SLOT(insertNode(KisNodeSP, KisNodeSP, int)));
-        disconnect(m_nodeModel, SIGNAL(requestMoveNode(KisNodeSP, KisNodeSP)), m_nodeManager, SLOT(moveNode(KisNodeSP, KisNodeSP)));
-        disconnect(m_nodeModel, SIGNAL(requestMoveNode(KisNodeSP, KisNodeSP, int)), m_nodeManager, SLOT(moveNodeAt(KisNodeSP, KisNodeSP, int)));
-    }
-    
-    m_image = image;
-
-    if (m_image && m_canvas && m_canvas->view()) {
-
-        if (m_nodeManager) {
-            m_nodeManager->disconnect(this);
-        }
         m_nodeManager = m_canvas->view()->nodeManager();
-        m_nodeModel->setImage(m_image);
+
+        KisDummiesFacadeBase *kritaDummiesFacade = dynamic_cast<KisDummiesFacadeBase*>(m_canvas->view()->document()->shapeController());
+        m_nodeModel->setDummiesFacade(kritaDummiesFacade, m_image);
+
         connect(m_image, SIGNAL(sigAboutToBeDeleted()), SLOT(notifyImageDeleted()));
 
         // cold start
         setCurrentNode(m_nodeManager->activeNode());
+
+        // Connection KisNodeManager -> KisLayerBox
         connect(m_nodeManager, SIGNAL(sigUiNeedChangeActiveNode(KisNodeSP)), this, SLOT(setCurrentNode(KisNodeSP)));
+
+        // Connection KisLayerBox -> KisNodeManager
+        // The order of these connections is important! See comment in the ctor
         connect(m_nodeModel, SIGNAL(nodeActivated(KisNodeSP)), m_nodeManager, SLOT(slotUiActivatedNode(KisNodeSP)));
+        connect(m_nodeModel, SIGNAL(nodeActivated(KisNodeSP)), SLOT(updateUI()));
 
-        connect(m_nodeModel, SIGNAL(requestAddNode(KisNodeSP, KisNodeSP)), m_nodeManager, SLOT(addNode(KisNodeSP, KisNodeSP)));
-        connect(m_nodeModel, SIGNAL(requestAddNode(KisNodeSP, KisNodeSP, int)), m_nodeManager, SLOT(insertNode(KisNodeSP, KisNodeSP, int)));
-        connect(m_nodeModel, SIGNAL(requestMoveNode(KisNodeSP, KisNodeSP)), m_nodeManager, SLOT(moveNode(KisNodeSP, KisNodeSP)));
-        connect(m_nodeModel, SIGNAL(requestMoveNode(KisNodeSP, KisNodeSP, int)), m_nodeManager, SLOT(moveNodeAt(KisNodeSP, KisNodeSP, int)));
-    }
-    else {
-        m_nodeModel->setImage(m_image);
+        // Node manipulation methods are forwarded to the node manager
+        connect(m_nodeModel, SIGNAL(requestAddNode(KisNodeSP, KisNodeSP, KisNodeSP)),
+                m_nodeManager, SLOT(addNodeDirect(KisNodeSP, KisNodeSP, KisNodeSP)));
+        connect(m_nodeModel, SIGNAL(requestMoveNode(KisNodeSP, KisNodeSP, KisNodeSP)),
+                m_nodeManager, SLOT(moveNodeDirect(KisNodeSP, KisNodeSP, KisNodeSP)));
+
+        m_wdgLayerBox->listLayers->expandAll();
+        expandNodesRecursively(m_image->rootLayer(), m_nodeModel, m_wdgLayerBox->listLayers);
+        m_wdgLayerBox->listLayers->scrollToBottom();
     }
 
-    m_wdgLayerBox->listLayers->expandAll();
-    m_wdgLayerBox->listLayers->scrollToBottom();
+}
+
+
+void KisLayerBox::unsetCanvas()
+{
+    setCanvas(0);
 }
 
 void KisLayerBox::notifyImageDeleted()
 {
-    setImage(0);
+    setCanvas(0);
 }
 
 void KisLayerBox::updateUI()
 {
-    KisNodeSP active = m_image ? m_nodeManager->activeNode() : 0;
+    if(!m_canvas) return;
+
+    KisNodeSP active = m_nodeManager->activeNode();
 
     m_wdgLayerBox->bnDelete->setEnabled(active);
     m_wdgLayerBox->bnRaise->setEnabled(active && (active->nextSibling()
@@ -316,33 +343,44 @@ void KisLayerBox::updateUI()
     m_wdgLayerBox->cmbComposite->setEnabled(active);
 
     if (active) {
-        if (m_nodeManager->activePaintDevice())
+        if (m_nodeManager->activePaintDevice()) {
             slotFillCompositeOps(m_nodeManager->activeColorSpace());
-        else
+        } else {
             slotFillCompositeOps(m_image->colorSpace());
+        }
+
         if (active->inherits("KisMask")) {
             active = active->parent(); // We need a layer to set opacity and composite op, which masks don't have
         }
         if (active->inherits("KisLayer")) {
             KisLayerSP l = qobject_cast<KisLayer*>(active.data());
             slotSetOpacity(l->opacity() * 100.0 / 255);
-            slotSetCompositeOp(l->compositeOp());
+
+            const KoCompositeOp* compositeOp = l->compositeOp();
+            if(compositeOp) {
+                slotSetCompositeOp(compositeOp);
+            } else {
+                m_wdgLayerBox->cmbComposite->setEnabled(false);
+            }
         }
     }
     m_newTransparencyMaskAction->setEnabled(active);
     m_newEffectMaskAction->setEnabled(active);
     m_newSelectionMaskAction->setEnabled(active);
     m_newCloneLayerAction->setEnabled(active);
-
 }
 
+
+/**
+ * This method is callen *only* when non-GUI code requested the
+ * change of the current node
+ */
 void KisLayerBox::setCurrentNode(KisNodeSP node)
 {
-    if (node) {
-        m_wdgLayerBox->listLayers->setCurrentIndex(m_nodeModel->indexFromNode(node));
-        m_nodeManager->activateNode(node);   // NOTE: temporary bug fix - Pentalis
-        updateUI();
-    }
+    QModelIndex index = node ? m_nodeModel->indexFromNode(node) : QModelIndex();
+
+    m_wdgLayerBox->listLayers->setCurrentIndex(index);
+    updateUI();
 }
 
 void KisLayerBox::slotSetCompositeOp(const KoCompositeOp* compositeOp)
@@ -390,8 +428,9 @@ void KisLayerBox::slotContextMenuRequested(const QPoint &pos, const QModelIndex
     menu.exec(pos);
 }
 
-void KisLayerBox::slotMergeLayer()      
+void KisLayerBox::slotMergeLayer()
 {
+    if(!m_canvas) return;
     m_nodeManager->mergeLayerDown();
 }
 
@@ -412,75 +451,73 @@ void KisLayerBox::slotThumbnailView()
 
 void KisLayerBox::slotNewPaintLayer()
 {
+    if(!m_canvas) return;
     m_nodeManager->createNode("KisPaintLayer");
 }
 
 void KisLayerBox::slotNewGroupLayer()
 {
+    if(!m_canvas) return;
     m_nodeManager->createNode("KisGroupLayer");
 }
 
 void KisLayerBox::slotNewCloneLayer()
 {
+    if(!m_canvas) return;
     m_nodeManager->createNode("KisCloneLayer");
 }
 
 
 void KisLayerBox::slotNewShapeLayer()
 {
+    if(!m_canvas) return;
     m_nodeManager->createNode("KisShapeLayer");
 }
 
 
 void KisLayerBox::slotNewAdjustmentLayer()
 {
+    if(!m_canvas) return;
     m_nodeManager->createNode("KisAdjustmentLayer");
 }
 
 void KisLayerBox::slotNewGeneratorLayer()
 {
+    if(!m_canvas) return;
     m_nodeManager->createNode("KisGeneratorLayer");
 }
 
 void KisLayerBox::slotNewTransparencyMask()
 {
+    if(!m_canvas) return;
     m_nodeManager->createNode("KisTransparencyMask");
 }
 
 void KisLayerBox::slotNewEffectMask()
 {
+    if(!m_canvas) return;
     m_nodeManager->createNode("KisFilterMask");
 }
 
 
 void KisLayerBox::slotNewSelectionMask()
 {
+    if(!m_canvas) return;
     m_nodeManager->createNode("KisSelectionMask");
 }
 
 
 void KisLayerBox::slotRmClicked()
 {
-    QModelIndexList l = m_wdgLayerBox->listLayers->selectionModel()->selectedIndexes();
-    if (l.count() < 2 && m_nodeManager->activeNode() && !l.contains(m_wdgLayerBox->listLayers->currentIndex())) {
-        l.clear();
-        l.append(m_wdgLayerBox->listLayers->currentIndex());
-    }
+    if(!m_canvas) return;
 
-    for (int i = 0, n = l.count(); i < n; ++i) {
-        KisNodeSP node = m_nodeModel->nodeFromIndex(l.at(i));
-        if (!node->systemLocked()) {
-            m_nodeManager->removeNode(node);
-        }
-    }
-    if (m_canvas && m_canvas->view()) {
-        KisView2* view = m_canvas->view();
-        view->updateGUI();
-    }
+    KisNodeSP node = m_nodeManager->activeNode();
+    m_nodeManager->removeNode(node);
 }
 
 void KisLayerBox::slotRaiseClicked()
 {
+    if(!m_canvas) return;
     KisNodeSP node = m_nodeManager->activeNode();
     KisNodeSP parent = node->parent();
     KisNodeSP grandParent = parent->parent();
@@ -496,6 +533,7 @@ void KisLayerBox::slotRaiseClicked()
 
 void KisLayerBox::slotLowerClicked()
 {
+    if(!m_canvas) return;
     KisNodeSP node = m_nodeManager->activeNode();
     KisNodeSP parent = node->parent();
     KisNodeSP grandParent = parent->parent();
@@ -511,6 +549,7 @@ void KisLayerBox::slotLowerClicked()
 
 void KisLayerBox::slotLeftClicked()
 {
+    if(!m_canvas) return;
     KisNodeSP node = m_nodeManager->activeNode();
     KisNodeSP parent = node->parent();
     KisNodeSP grandParent = parent->parent();
@@ -528,6 +567,7 @@ void KisLayerBox::slotLeftClicked()
 
 void KisLayerBox::slotRightClicked()
 {
+    if(!m_canvas) return;
     KisNodeSP node = m_nodeManager->activeNode();
     KisNodeSP parent = m_nodeManager->activeNode()->parent();
     KisNodeSP newParent;
@@ -548,6 +588,7 @@ void KisLayerBox::slotRightClicked()
 
 void KisLayerBox::slotPropertiesClicked()
 {
+    if(!m_canvas) return;
     if (KisNodeSP active = m_nodeManager->activeNode()) {
         m_nodeManager->nodeProperties(active);
     }
@@ -555,19 +596,22 @@ void KisLayerBox::slotPropertiesClicked()
 
 void KisLayerBox::slotDuplicateClicked()
 {
+    if(!m_canvas) return;
     m_nodeManager->duplicateActiveNode();
 }
 
 void KisLayerBox::slotCompositeOpChanged(int index)
 {
+    if(!m_canvas) return;
+
     KoID compositeOp;
-    
     if(m_wdgLayerBox->cmbComposite->entryAt(compositeOp, index))
         m_nodeManager->nodeCompositeOpChanged(m_nodeManager->activeColorSpace()->compositeOp(compositeOp.id()));
 }
 
 void KisLayerBox::slotOpacityChanged()
 {
+    if(!m_canvas) return;
     m_nodeManager->nodeOpacityChanged(m_newOpacity, true);
 }
 
@@ -577,6 +621,21 @@ void KisLayerBox::slotOpacitySliderMoved(qreal opacity)
     m_delayTimer.start(200);
 }
 
+void KisLayerBox::slotCollapsed(const QModelIndex &index)
+{
+    KisNodeSP node = m_nodeModel->nodeFromIndex(index);
+    if (node) {
+        node->setCollapsed(true);
+    }
+}
+
+void KisLayerBox::slotExpanded(const QModelIndex &index)
+{
+    KisNodeSP node = m_nodeModel->nodeFromIndex(index);
+    if (node) {
+        node->setCollapsed(false);
+    }
+}
 
 
 #include "kis_layer_box.moc"
