@@ -86,7 +86,7 @@ KisPDFImport::ConversionStatus KisPDFImport::convert(const QByteArray& , const Q
         url.setPath(tmpFile);
     }
 
-    Poppler::Document* pdoc = Poppler::Document::load(QFile::encodeName(url.toLocalFile()));
+    Poppler::Document* pdoc = Poppler::Document::load(url.toLocalFile());
     pdoc->setRenderHint(Poppler::Document::Antialiasing, true);
     pdoc->setRenderHint(Poppler::Document::TextAntialiasing, true);
 
@@ -144,13 +144,21 @@ KisPDFImport::ConversionStatus KisPDFImport::convert(const QByteArray& , const Q
         KisTransaction(0, layer->paintDevice());
 
         Poppler::Page* page = pdoc->page(*it);
-        for (int x = 0; x < width; x += 1000) {
-            int currentWidth = (x + 1000 > width) ? (width - x) : 1000;
-            for (int y = 0; y < height; y += 1000) {
-                int currentHeight = (y + 1000 > height) ? (height - x) : 1000;
-                layer->paintDevice()->convertFromQImage(page->renderToImage(wdg->intHorizontal->value(), wdg->intVertical->value(), x, y, currentWidth, currentHeight), 0, x, y);
-            }
-        }
+
+        QImage img = page->renderToImage(wdg->intHorizontal->value(), wdg->intVertical->value(), 0, 0, width, height);
+        layer->paintDevice()->convertFromQImage(img, 0, 0, 0);
+
+// XXX: this rendering in tiles is a good idea, but: a) it is slower b) it is buggy -- see bug https://bugs.kde.org/show_bug.cgi?id=300554
+
+//        for (int x = 0; x < width; x += 1000) {
+//            int currentWidth = (x + 1000 > width) ? (width - x) : 1000;
+//            for (int y = 0; y < height; y += 1000) {
+//                int currentHeight = (y + 1000 > height) ? (height - x) : 1000;
+//                qDebug() << wdg->intHorizontal->value() << wdg->intVertical->value() << x << y << currentWidth << currentHeight;
+//                QImage img = page->renderToImage(wdg->intHorizontal->value(), wdg->intVertical->value(), x, y, currentWidth, currentHeight);
+//                layer->paintDevice()->convertFromQImage(img, 0, x, y);
+//            }
+//        }
         delete page;
         image->addNode(layer, image->rootLayer(), 0);
     }
