@@ -97,6 +97,8 @@ KisLayerManager::KisLayerManager(KisView2 * view, KisDoc2 * doc)
         , m_imageResizeToLayer(0)
         , m_flattenLayer(0)
         , m_rasterizeLayer(0)
+        , m_duplicateLayer(0)
+        , m_addPaintLayer(0)
         , m_activeLayer(0)
         , m_commandsAdapter(new KisNodeCommandsAdapter(m_view))
 {
@@ -158,6 +160,16 @@ void KisLayerManager::setup(KActionCollection * actionCollection)
     m_imageResizeToLayer  = new KAction(i18n("Size Canvas to Size of Current Layer"), this);
     actionCollection->addAction("resizeimagetolayer", m_imageResizeToLayer);
     connect(m_imageResizeToLayer, SIGNAL(triggered()), this, SLOT(imageResizeToActiveLayer()));
+
+    m_duplicateLayer = new KAction(i18n("Duplicate current layer"), this);
+    m_duplicateLayer->setShortcut(KShortcut(Qt::ControlModifier + Qt::Key_J));
+    actionCollection->addAction("duplicatelayer", m_duplicateLayer);
+    connect(m_duplicateLayer, SIGNAL(triggered()), this, SLOT(layerDuplicate()));
+
+    m_addPaintLayer = new KAction(i18n("Add new paint layer"), this);
+    m_addPaintLayer->setShortcut(KShortcut(Qt::Key_Insert));
+    actionCollection->addAction("add_new_paint_layer", m_addPaintLayer);
+    connect(m_addPaintLayer, SIGNAL(triggered()), this, SLOT(layerAdd()));
 }
 
 void KisLayerManager::addAction(QAction * action)
@@ -273,7 +285,7 @@ void KisLayerManager::layerProperties()
             }
 
         }
-    } else { // If layer == normal painting layer, shape layer, or group layer
+    } else { // If layer == normal painting layer, vector layer, or group layer
         KisDlgLayerProperties *dialog = new KisDlgLayerProperties(layer, m_view, m_doc);
         dialog->resize(dialog->minimumSizeHint());
         dialog->setAttribute(Qt::WA_DeleteOnClose);
