@@ -40,6 +40,7 @@
 #include <klocale.h>
 #include <kiconloader.h>
 #include <kactioncollection.h>
+#include <kaction.h>
 
 #include <KoShape.h>
 #include <KoShapeManager.h>
@@ -58,21 +59,20 @@
 #include <kis_layer.h>
 #include <kis_view2.h>
 #include <kis_canvas2.h>
+#include <kis_cubic_curve.h>
 
 #include "kis_config.h"
 #include "kis_config_notifier.h"
-
 #include "kis_cursor.h"
 #include "widgets/kis_cmb_composite.h"
 #include "widgets/kis_slider_spin_box.h"
 #include "kis_canvas_resource_provider.h"
 #include <recorder/kis_recorded_paint_action.h>
-#include <kis_cubic_curve.h>
 #include "kis_color_picker_utils.h"
 #include <kis_paintop.h>
-#include <kaction.h>
+#include "kis_canvas_resource_provider.h"
 
-const int STEP = 20;
+const int STEP = 25;
 
 KisToolPaint::KisToolPaint(KoCanvasBase * canvas, const QCursor & cursor)
     : KisTool(canvas, cursor)
@@ -88,14 +88,14 @@ KisToolPaint::KisToolPaint(KoCanvasBase * canvas, const QCursor & cursor)
 
     KActionCollection *collection = this->canvas()->canvasController()->actionCollection();
 
-    if(!collection->action("make_brush_color_lighter")) {
-        KAction *lighterColor = new KAction(i18n("Make Brush color lighter"), collection);
+    if (!collection->action("make_brush_color_lighter")) {
+        KAction *lighterColor = new KAction(i18n("Make brush color lighter"), collection);
         lighterColor->setShortcut(Qt::Key_L);
         collection->addAction("make_brush_color_lighter", lighterColor);
     }
 
-    if(!collection->action("make_brush_color_darker")) {
-        KAction *darkerColor = new KAction(i18n("Make Brush color darker"), collection);
+    if (!collection->action("make_brush_color_darker")) {
+        KAction *darkerColor = new KAction(i18n("Make brush color darker"), collection);
         darkerColor->setShortcut(Qt::Key_K);
         collection->addAction("make_brush_color_darker", darkerColor);
     }
@@ -103,6 +103,22 @@ KisToolPaint::KisToolPaint(KoCanvasBase * canvas, const QCursor & cursor)
     addAction("make_brush_color_lighter", dynamic_cast<KAction*>(collection->action("make_brush_color_lighter")));
     addAction("make_brush_color_darker", dynamic_cast<KAction*>(collection->action("make_brush_color_darker")));
 
+    if (!collection->action("increase_opacity")) {
+        KAction *increaseOpacity = new KAction(i18n("Increase opacity"), collection);
+        increaseOpacity->setShortcut(Qt::Key_O);
+        collection->addAction("increase_opacity", increaseOpacity);
+    }
+
+    if (!collection->action("decrease_opacity")) {
+        KAction *increaseOpacity = new KAction(i18n("Decrease opacity"), collection);
+        increaseOpacity->setShortcut(Qt::Key_I);
+        collection->addAction("decrease_opacity", increaseOpacity);
+    }
+
+    addAction("decrease_opacity", dynamic_cast<KAction*>(collection->action("decrease_opacity")));
+    addAction("increase_opacity", dynamic_cast<KAction*>(collection->action("increase_opacity")));
+
+
     KisCanvas2 * kiscanvas = static_cast<KisCanvas2*>(canvas);
     connect(this, SIGNAL(sigFavoritePaletteCalled(const QPoint&)), kiscanvas, SIGNAL(favoritePaletteCalled(const QPoint&)));
     connect(this, SIGNAL(sigPaintingFinished()), kiscanvas->view()->resourceProvider(), SLOT(slotPainting()));
@@ -130,8 +146,8 @@ void KisToolPaint::resourceChanged(int key, const QVariant& v)
         break;
     }
 
-    connect(KisConfigNotifier::instance(), SIGNAL(configChanged()), SLOT(resetCursorStyle()));
-    connect(KisConfigNotifier::instance(), SIGNAL(configChanged()), SLOT(updateTabletPressureSamples()));
+    connect(KisConfigNotifier::instance(), SIGNAL(configChanged()), SLOT(resetCursorStyle()), Qt::UniqueConnection);
+    connect(KisConfigNotifier::instance(), SIGNAL(configChanged()), SLOT(updateTabletPressureSamples()), Qt::UniqueConnection);
 
 }
 
@@ -139,14 +155,18 @@ void KisToolPaint::resourceChanged(int key, const QVariant& v)
 void KisToolPaint::activate(ToolActivation toolActivation, const QSet<KoShape*> &shapes)
 {
     KisTool::activate(toolActivation, shapes);
-    connect(actions().value("make_brush_color_lighter"), SIGNAL(triggered()), SLOT(makeColorLighter()));
-    connect(actions().value("make_brush_color_darker"), SIGNAL(triggered()), SLOT(makeColorDarker()));
+    connect(actions().value("make_brush_color_lighter"), SIGNAL(triggered()), SLOT(makeColorLighter()), Qt::UniqueConnection);
+    connect(actions().value("make_brush_color_darker"), SIGNAL(triggered()), SLOT(makeColorDarker()), Qt::UniqueConnection);
+    connect(actions().value("increase_opacity"), SIGNAL(triggered()), SLOT(increaseOpacity()), Qt::UniqueConnection);
+    connect(actions().value("decrease_opacity"), SIGNAL(triggered()), SLOT(decreaseOpacity()), Qt::UniqueConnection);
 }
 
 void KisToolPaint::deactivate()
 {
     disconnect(actions().value("make_brush_color_lighter"), 0, this, 0);
     disconnect(actions().value("make_brush_color_darker"), 0, this, 0);
+    disconnect(actions().value("increase_opacity"), 0, this, 0);
+    disconnect(actions().value("decrease_opacity"), 0, this, 0);
     KisTool::deactivate();
 }
 
@@ -169,15 +189,7 @@ void KisToolPaint::setMode(ToolMode mode)
 
 void KisToolPaint::mousePressEvent(KoPointerEvent *event)
 {
-    if(mode() == KisTool::HOVER_MODE &&
-            (event->button() == Qt::LeftButton) &&
-            event->modifiers() == (Qt::ControlModifier | Qt::ShiftModifier) &&
-            !specialModifierActive()) {
-        setMode(MIRROR_AXIS_SETUP_MODE);
-        useCursor(KisCursor::crossCursor());
-        canvas()->resourceManager()->setResource(KisCanvasResourceProvider::MirrorAxisCenter, convertToPixelCoord(event->point));
-    }
-    else if((event->button() == Qt::LeftButton || event->button() == Qt::RightButton) &&
+    if((event->button() == Qt::LeftButton || event->button() == Qt::RightButton) &&
             event->modifiers() & Qt::ControlModifier &&
             !specialModifierActive()) {
 
@@ -189,14 +201,6 @@ void KisToolPaint::mousePressEvent(KoPointerEvent *event)
                   m_toForegroundColor);
         event->accept();
     }
-    else if(mode() == KisTool::HOVER_MODE &&
-            event->button() == Qt::RightButton &&
-            event->modifiers() == Qt::NoModifier &&
-            !specialModifierActive()) {
-
-        emit sigFavoritePaletteCalled(event->pos());
-        event->accept();
-    }
     else {
         KisTool::mousePressEvent(event);
     }
@@ -209,9 +213,6 @@ void KisToolPaint::mouseMoveEvent(KoPointerEvent *event)
                   m_toForegroundColor);
         event->accept();
     }
-    else if (mode() == KisTool::MIRROR_AXIS_SETUP_MODE){
-        canvas()->resourceManager()->setResource(KisCanvasResourceProvider::MirrorAxisCenter, convertToPixelCoord(event->point));
-    }
     else {
         KisTool::mouseMoveEvent(event);
     }
@@ -219,11 +220,7 @@ void KisToolPaint::mouseMoveEvent(KoPointerEvent *event)
 
 void KisToolPaint::mouseReleaseEvent(KoPointerEvent *event)
 {
-    if(mode() == KisTool::MIRROR_AXIS_SETUP_MODE) {
-        setMode(KisTool::HOVER_MODE);
-        resetCursorStyle();
-        event->accept();
-    } else if(mode() == KisTool::SECONDARY_PAINT_MODE) {
+    if(mode() == KisTool::SECONDARY_PAINT_MODE) {
         setMode(KisTool::HOVER_MODE);
         if(!(event->modifiers() == Qt::ControlModifier)) {
             resetCursorStyle();
@@ -237,19 +234,11 @@ void KisToolPaint::mouseReleaseEvent(KoPointerEvent *event)
 void KisToolPaint::keyPressEvent(QKeyEvent *event)
 {
     if (mode() == KisTool::HOVER_MODE &&
-        (event->key() == Qt::Key_Control || event->key() == Qt::Key_Shift) &&
-        (event->modifiers() == (Qt::ShiftModifier | Qt::ControlModifier))) {
-        useCursor(KisCursor::crossCursor());
-        m_specialHoverModifier = true;
-        event->accept();
-    } else if (mode() == KisTool::HOVER_MODE &&
                event->key() == Qt::Key_Control) {
         useCursor(KisCursor::pickerCursor());
         m_specialHoverModifier = true;
         event->accept();
-    } else if (mode() == KisTool::SECONDARY_PAINT_MODE ||
-               mode() == KisTool::MIRROR_AXIS_SETUP_MODE) {
-
+    } else if (mode() == KisTool::SECONDARY_PAINT_MODE) {
         event->accept();
     } else {
         KisTool::keyPressEvent(event);
@@ -258,23 +247,16 @@ void KisToolPaint::keyPressEvent(QKeyEvent *event)
 
 void KisToolPaint::keyReleaseEvent(QKeyEvent* event)
 {
-    bool mirrorCondition =
-        (event->key() == Qt::Key_Control && event->modifiers() == Qt::ShiftModifier) ||
-        (event->key() == Qt::Key_Shift && event->modifiers() == Qt::ControlModifier);
-
     bool pickerCondition =
         event->key() == Qt::Key_Control;
 
-    if(mirrorCondition || pickerCondition) {
+    if(pickerCondition) {
         m_specialHoverModifier = false;
-        if(mode() != KisTool::SECONDARY_PAINT_MODE &&
-           mode() != KisTool::MIRROR_AXIS_SETUP_MODE) {
+        if(mode() != KisTool::SECONDARY_PAINT_MODE) {
             resetCursorStyle();
             event->accept();
         }
-    } else if (mode() == KisTool::SECONDARY_PAINT_MODE ||
-               mode() == KisTool::MIRROR_AXIS_SETUP_MODE) {
-
+    } else if (mode() == KisTool::SECONDARY_PAINT_MODE) {
         event->accept();
     } else {
         KisTool::keyReleaseEvent(event);
@@ -473,6 +455,7 @@ void KisToolPaint::transformColor(int step)
     canvas()->resourceManager()->setResource(KoCanvasResourceManager::ForegroundColor, color);
 }
 
+
 void KisToolPaint::makeColorDarker()
 {
     transformColor(-STEP);
@@ -484,4 +467,24 @@ void KisToolPaint::makeColorLighter()
 }
 
 
+void KisToolPaint::stepAlpha(float step)
+{
+    qreal alpha = canvas()->resourceManager()->resource(KisCanvasResourceProvider::Opacity).toDouble();
+    alpha += step;
+    alpha = qBound(0.0, alpha, 1.0);
+    canvas()->resourceManager ()->setResource(KisCanvasResourceProvider::Opacity, alpha);
+}
+
+
+void KisToolPaint::increaseOpacity()
+{
+    stepAlpha(0.1);
+}
+
+void KisToolPaint::decreaseOpacity()
+{
+    stepAlpha(-0.1);
+}
+
+
 #include "kis_tool_paint.moc"
