@@ -249,7 +249,15 @@ void KoParagraphStyle::applyStyle(QTextBlockFormat &format) const
     const QMap<int, QVariant> props = d->stylesPrivate.properties();
     QMap<int, QVariant>::const_iterator it = props.begin();
     while (it != props.end()) {
-        format.setProperty(it.key(), it.value());
+        if (it.key() == QTextBlockFormat::BlockLeftMargin) {
+            format.setLeftMargin(leftMargin());
+        } else if (it.key() == QTextBlockFormat::BlockRightMargin) {
+            format.setRightMargin(rightMargin());
+        } else if (it.key() == QTextBlockFormat::TextIndent) {
+            format.setTextIndent(textIndent());
+        } else {
+            format.setProperty(it.key(), it.value());
+        }
         ++it;
     }
     if ((hasProperty(DefaultOutlineLevel)) && (!format.hasProperty(OutlineLevel))) {
@@ -268,41 +276,48 @@ void KoParagraphStyle::applyStyle(QTextBlock &block, bool applyListStyle) const
     KoCharacterStyle::applyStyle(block);
 
     if (applyListStyle) {
-        //gopalakbhat: We need to differentiate between normal styles and styles with outline(part of heading)
-        //Styles part of outline: We ignore the listStyle()( even if this is a valid in ODF; even LibreOffice does the same)
-        //                        since we can specify all info required in text:outline-style
-        //Normal styles: we use the listStyle()
-        if (format.hasProperty(OutlineLevel)) {
-            if (! d->list) {
-                if (! KoTextDocument(block.document()).headingList()) {
-                    if (KoTextDocument(block.document()).styleManager() && KoTextDocument(block.document()).styleManager()->outlineStyle()) {
-                        d->list = new KoList(block.document(), KoTextDocument(block.document()).styleManager()->outlineStyle());
-                        KoTextDocument(block.document()).setHeadingList(d->list);
-                    }
-                } else {
-                    d->list = KoTextDocument(block.document()).headingList();
-                }
-            }
-            if (d->list) {
-                d->list->applyStyle(block, KoTextDocument(block.document()).styleManager()->outlineStyle(), format.intProperty(OutlineLevel));
-            }
-        } else {
-            if (listStyle()) {
-                if (!d->list) {
-                    d->list = new KoList(block.document(), listStyle());
-                }
-                if (format.hasProperty(OutlineLevel)) {
+        applyParagraphListStyle(block, format);
+    }
+}
 
+void KoParagraphStyle::applyParagraphListStyle(QTextBlock &block, const QTextBlockFormat &blockFormat) const
+{
+    //gopalakbhat: We need to differentiate between normal styles and styles with outline(part of heading)
+    //Styles part of outline: We ignore the listStyle()( even if this is a valid in ODF; even LibreOffice does the same)
+    //                        since we can specify all info required in text:outline-style
+    //Normal styles: we use the listStyle()
+    if (blockFormat.hasProperty(OutlineLevel)) {
+        if (! d->list) {
+            if (! KoTextDocument(block.document()).headingList()) {
+                if (KoTextDocument(block.document()).styleManager() && KoTextDocument(block.document()).styleManager()->outlineStyle()) {
+                    d->list = new KoList(block.document(), KoTextDocument(block.document()).styleManager()->outlineStyle());
+                    KoTextDocument(block.document()).setHeadingList(d->list);
                 }
-                d->list->add(block, listLevel());
             } else {
-                if (block.textList())
-                    block.textList()->remove(block);
-                KoTextBlockData *data = dynamic_cast<KoTextBlockData*>(block.userData());
-                if (data)
-                    data->setCounterWidth(-1);
+                d->list = KoTextDocument(block.document()).headingList();
             }
         }
+        if (d->list) {
+            d->list->applyStyle(block, KoTextDocument(block.document()).styleManager()->outlineStyle(), blockFormat.intProperty(OutlineLevel));
+        }
+    } else {
+        if (listStyle()) {
+            if (!d->list) {
+                d->list = new KoList(block.document(), listStyle());
+            }
+            //FIXME: Gopalakrishna Bhat A: This condition should never happen.
+            // i.e. d->list->style() should always be in sync with the listStyle()
+            if (d->list->style() != listStyle()) {
+                d->list->setStyle(listStyle());
+            }
+            d->list->add(block, listLevel());
+        } else {
+            if (block.textList())
+                block.textList()->remove(block);
+            KoTextBlockData *data = dynamic_cast<KoTextBlockData*>(block.userData());
+            if (data)
+                data->setCounterWidth(-1);
+        }
     }
 }
 
@@ -317,13 +332,32 @@ void KoParagraphStyle::unapplyStyle(QTextBlock &block) const
     QList<int> keys = d->stylesPrivate.keys();
     for (int i = 0; i < keys.count(); i++) {
         QVariant variant = d->stylesPrivate.value(keys[i]);
-        if (variant == format.property(keys[i]))
-            format.clearProperty(keys[i]);
+        if (keys[i] == QTextBlockFormat::BlockLeftMargin) {
+            if (leftMargin() == format.property(keys[i]))
+                format.clearProperty(keys[i]);
+        } else if (keys[i] == QTextBlockFormat::BlockRightMargin) {
+            if (rightMargin() == format.property(keys[i]))
+                format.clearProperty(keys[i]);
+        } else if (keys[i] == QTextBlockFormat::TextIndent) {
+            if (textIndent() == format.property(keys[i]))
+                format.clearProperty(keys[i]);
+        } else {
+            if (variant == format.property(keys[i]))
+                format.clearProperty(keys[i]);
+        }
     }
+
+    format.clearProperty(KoParagraphStyle::OutlineLevel);
+
     cursor.setBlockFormat(format);
     KoCharacterStyle::unapplyStyle(block);
-    if (listStyle() && block.textList()) // TODO check its the same one?
-        block.textList()->remove(block);
+    if (listStyle() && block.textList()) { // TODO check its the same one?
+        KoList::remove(block);
+    }
+    if (d->list && block.textList()) { // TODO check its the same one?
+        KoList::remove(block);
+    }
+
 }
 
 void KoParagraphStyle::setLineHeightPercent(int lineHeight)
@@ -840,9 +874,12 @@ void KoParagraphStyle::setTextIndent(QTextLength margin)
     setProperty(QTextFormat::TextIndent, margin);
 }
 
-QTextLength KoParagraphStyle::textIndent() const
+qreal KoParagraphStyle::textIndent() const
 {
-    return propertyLength(QTextFormat::TextIndent);
+    if (parentStyle())
+        return propertyLength(QTextFormat::TextIndent).value(parentStyle()->textIndent());
+    else
+        return propertyLength(QTextFormat::TextIndent).value(0);
 }
 
 void KoParagraphStyle::setAutoTextIndent(bool on)
@@ -1038,8 +1075,15 @@ void KoParagraphStyle::setListStyle(KoListStyle *style)
     if (listStyle() && listStyle()->parent() == this)
         delete listStyle();
     QVariant variant;
-    variant.setValue(style->clone());
-    setProperty(ParagraphListStyleId, variant);
+    KoListStyle *cloneStyle = 0;
+    if (style) {
+        cloneStyle = style->clone();
+        variant.setValue(cloneStyle);
+
+        setProperty(ParagraphListStyleId, variant);
+    } else {
+        d->stylesPrivate.remove(ParagraphListStyleId);
+    }
 }
 
 KoText::Direction KoParagraphStyle::textProgressionDirection() const
@@ -1214,18 +1258,7 @@ void KoParagraphStyle::loadOdf(const KoXmlElement *element, KoShapeLoadingContex
             setDefaultOutlineLevel(level);
     }
 
-    //1.6: KoTextParag::loadOasis => KoParagLayout::loadOasisParagLayout
-    kDebug(32500) << "family" << family;
-    if (family == "graphic" || family == "presentation") {
-        QList<QString> typeProperties;
-        typeProperties.append("graphic");
-        typeProperties.append("paragraph");
-        // load all style attributes from "style::graphic-properties" and if not found there from "style:paragraph-properties"
-        context.styleStack().setTypeProperties(typeProperties);
-    }
-    else {
-        context.styleStack().setTypeProperties("paragraph");   // load all style attributes from "style:paragraph-properties"
-    }
+    context.styleStack().setTypeProperties("paragraph");   // load all style attributes from "style:paragraph-properties"
 
     loadOdfProperties(scontext);   // load the KoParagraphStyle from the stylestack
 
@@ -1573,9 +1606,13 @@ void KoParagraphStyle::loadOdfProperties(KoShapeLoadingContext &scontext)
         setDropCaps(true);
         const QString length = dropCap.attributeNS(KoXmlNS::style, "length", QString("1"));
         if (length.toLower() == "word") {
-            setDropCapsLength(-1); // -1 indicates drop caps of the whole first word
+            setDropCapsLength(0); // 0 indicates drop caps of the whole first word
         } else {
-            setDropCapsLength(length.toInt());
+            int l = length.toInt();
+            if (l > 0) // somefiles may use this to turn dropcaps off
+                setDropCapsLength(length.toInt());
+            else 
+                setDropCaps(false);
         }
         const QString lines = dropCap.attributeNS(KoXmlNS::style, "lines", QString("1"));
         setDropCapsLines(lines.toInt());
@@ -1843,9 +1880,10 @@ void KoParagraphStyle::copyProperties(const KoParagraphStyle *style)
     setName(style->name()); // make sure we emit property change
     KoCharacterStyle::copyProperties(style);
     d->parentStyle = style->d->parentStyle;
+    d->defaultStyle = style->d->defaultStyle;
 }
 
-KoParagraphStyle *KoParagraphStyle::clone(QObject *parent)
+KoParagraphStyle *KoParagraphStyle::clone(QObject *parent) const
 {
     KoParagraphStyle *newStyle = new KoParagraphStyle(parent);
     newStyle->copyProperties(this);
@@ -2023,7 +2061,7 @@ void KoParagraphStyle::saveOdf(KoGenStyle &style, KoShapeSavingContext &context)
             }
     //
         } else if (key == QTextFormat::TextIndent) {
-            style.addPropertyLength("fo:text-indent", textIndent(), KoGenStyle::ParagraphType);
+            style.addPropertyLength("fo:text-indent", propertyLength(QTextFormat::TextIndent), KoGenStyle::ParagraphType);
         } else if (key == KoParagraphStyle::AutoTextIndent) {
             style.addProperty("style:auto-text-indent", autoTextIndent(), KoGenStyle::ParagraphType);
         } else if (key == KoParagraphStyle::TabStopDistance) {
@@ -2143,12 +2181,12 @@ void KoParagraphStyle::saveOdf(KoGenStyle &style, KoShapeSavingContext &context)
         KoXmlWriter elementWriter(&buf, indentation);
         elementWriter.startElement("style:drop-cap");
         elementWriter.addAttribute("style:lines", QString::number(dropCapsLines()));
-        elementWriter.addAttribute("style:length", dropCapsLength() < 0 ? "word" : QString::number(dropCapsLength()));
+        elementWriter.addAttribute("style:length", dropCapsLength() == 0 ? "word" : QString::number(dropCapsLength()));
         if (dropCapsDistance())
             elementWriter.addAttributePt("style:distance", dropCapsDistance());
         elementWriter.endElement();
         QString elementContents = QString::fromUtf8(buf.buffer(), buf.buffer().size());
-        style.addChildElement("style:drop-cap", elementContents);
+        style.addChildElement("style:drop-cap", elementContents, KoGenStyle::ParagraphType);
     }
     if (tabPositions().count() > 0) {
         QMap<int, QString> tabTypeMap, leaderTypeMap, leaderStyleMap, leaderWeightMap;
@@ -2207,7 +2245,7 @@ void KoParagraphStyle::saveOdf(KoGenStyle &style, KoShapeSavingContext &context)
         elementWriter.endElement();
         buf.close();
         QString elementContents = QString::fromUtf8(buf.buffer(), buf.buffer().size());
-        style.addChildElement("style:tab-stops", elementContents);
+        style.addChildElement("style:tab-stops", elementContents, KoGenStyle::ParagraphType);
     }
 }
 
@@ -2220,4 +2258,9 @@ bool KoParagraphStyle::hasDefaults() const
     return false;
 }
 
+KoList *KoParagraphStyle::list()
+{
+    return d->list;
+}
+
 #include <KoParagraphStyle.moc>
