@@ -33,6 +33,7 @@
 #include "KoParagraphStyle.h"
 
 #include <KDebug>
+#include <KoTextDebug.h>
 
 #include <QTextTable>
 #include <QTextTableFormat>
@@ -111,6 +112,28 @@ KoTableCellStyle *KoTableCellStyle::fromTableCell(const QTextTableCell &tableCel
     return new KoTableCellStyle(tableCellFormat, parent);
 }
 
+QTextCharFormat KoTableCellStyle::cleanCharFormat(const QTextCharFormat &charFormat)
+{
+    if (charFormat.isTableCellFormat()) {
+        QTextTableCellFormat format;
+        const QMap<int, QVariant> props = charFormat.properties();
+        QMap<int, QVariant>::const_iterator it = props.begin();
+        while (it != props.end()) {
+            // lets save all Qt's table cell properties
+            if (it.key()>=QTextFormat::TableCellRowSpan && it.key()<QTextFormat::ImageName)
+                format.setProperty(it.key(), it.value());
+
+            // lets save all our table cell properties
+            if (it.key()>=StyleId && it.key()<LastCellStyleProperty)
+                format.setProperty(it.key(), it.value());
+
+            ++it;
+        }
+        return format;
+    }
+    return QTextCharFormat();
+}
+
 QRectF KoTableCellStyle::contentRect(const QRectF &boundingRect) const
 {
     const KoBorder::BorderData &leftEdge = getEdge(KoBorder::Left);
@@ -327,7 +350,6 @@ void KoTableCellStyle::applyStyle(QTextTableCell &cell) const
         d->paragraphStyle->KoCharacterStyle::applyStyle(format);
     }
     cell.setFormat(format);
-    //FIXME TODO we should apply the paragrapStyle under all contained blocks
 }
 
 void KoTableCellStyle::setBackground(const QBrush &brush)
@@ -943,24 +965,26 @@ void KoTableCellStyle::setEdge(KoBorder::Side side, KoBorder::BorderStyle style,
     qreal innerWidth = 0;
     qreal middleWidth = 0;
     qreal space = 0;
+    QVector<qreal> dashes;
     switch (style) {
     case KoBorder::BorderNone:
         width = 0.0;
         break;
     case KoBorder::BorderDouble:
-        innerWidth = space = width/4; //some nice default look
+        innerWidth = space = width/3; //some nice default look
         width -= (space + innerWidth);
         edge.outerPen.setStyle(Qt::SolidLine);
         break;
     case KoBorder::BorderDotted:
-        edge.outerPen.setStyle(Qt::DotLine);
+        dashes << 1 << 1;
+        edge.outerPen.setDashPattern(dashes);
         break;
     case KoBorder::BorderDashed:
-        edge.outerPen.setStyle(Qt::DashLine);
+        dashes << 4 << 1;
+        edge.outerPen.setDashPattern(dashes);
         break;
     case KoBorder::BorderDashedLong: {
-        QVector<qreal> dashes;
-        dashes << 6 << 6;
+        dashes << 4 << 4;
         edge.outerPen.setDashPattern(dashes);
         break;
     }
@@ -970,10 +994,12 @@ void KoTableCellStyle::setEdge(KoBorder::Side side, KoBorder::BorderStyle style,
         edge.outerPen.setStyle(Qt::SolidLine);
         break;
     case KoBorder::BorderDashDot:
-        edge.outerPen.setStyle(Qt::DashDotLine);
+        dashes << 3 << 3<< 7 << 3;
+        edge.outerPen.setDashPattern(dashes);
         break;
     case KoBorder::BorderDashDotDot:
-        edge.outerPen.setStyle(Qt::DashDotDotLine);
+        dashes << 2 << 2<< 6 << 2 << 2 << 2;
+        edge.outerPen.setDashPattern(dashes);
         break;
     case KoBorder::BorderWave:
         edge.outerPen.setStyle(Qt::SolidLine);
@@ -982,7 +1008,7 @@ void KoTableCellStyle::setEdge(KoBorder::Side side, KoBorder::BorderStyle style,
         edge.outerPen.setStyle(Qt::SolidLine);
         break;
     case KoBorder::BorderDoubleWave:
-        innerWidth = space = width/4; //some nice default look
+        innerWidth = space = width/3; //some nice default look
         width -= (space + innerWidth);
         edge.outerPen.setStyle(Qt::SolidLine);
         break;
