@@ -2,7 +2,7 @@
  * Copyright (C) 2006, 2009-2010 Thomas Zander <zander@kde.org>
  * Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
  * Copyright (C) 2008 Girish Ramakrishnan <girish@forwardbias.in>
- * Copyright (C) 2011 Casper Boemann <cbo@boemann.dk>
+ * Copyright (C) 2011 C. Boemann <cbo@boemann.dk>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -67,6 +67,10 @@ class KoTextShapeDataPrivate : public KoTextShapeDataBasePrivate
 public:
     KoTextShapeDataPrivate()
             : ownsDocument(true)
+            , topPadding(0)
+            , leftPadding(0)
+            , rightPadding(0)
+            , bottomPadding(0)
             , direction(KoText::AutoDirection)
             , textpage(0)
             , rootArea(0)
@@ -84,6 +88,10 @@ public:
     }
 
     bool ownsDocument;
+    qreal topPadding;
+    qreal leftPadding;
+    qreal rightPadding;
+    qreal bottomPadding;
     KoText::Direction direction;
     KoTextPage *textpage;
     KoTextLayoutRootArea *rootArea;
@@ -108,27 +116,33 @@ void KoTextShapeData::setDocument(QTextDocument *document, bool transferOwnershi
     if (d->ownsDocument && document != d->document)
         delete d->document;
     d->ownsDocument = transferOwnership;
-    if (d->document == document)
-        return;
-    d->document = document;
+
     // The following avoids the normal case where the glyph metrices are rounded to integers and
     // hinted to the screen by freetype, which you of course don't want for WYSIWYG
-    if (! d->document->useDesignMetrics())
-        d->document->setUseDesignMetrics(true);
+    if (! document->useDesignMetrics())
+        document->setUseDesignMetrics(true);
 
-    if (d->document->isEmpty() && !d->document->firstBlock().blockFormat().hasProperty(KoParagraphStyle::StyleId)) { // apply app default style for first parag
-        KoTextDocument doc(d->document);
-        KoStyleManager *sm = doc.styleManager();
+    KoTextDocument kodoc(document);
+
+    if (document->isEmpty() && !document->firstBlock().blockFormat().hasProperty(KoParagraphStyle::StyleId)) { // apply app default style for first parag
+        KoStyleManager *sm = kodoc.styleManager();
         if (sm) {
             KoParagraphStyle *defaultStyle = sm->defaultParagraphStyle();
             if (defaultStyle) {
-                QTextBlock block = d->document->begin();
+                QTextBlock block = document->begin();
                 defaultStyle->applyStyle(block);
             }
         }
     }
 
-    KoTextDocument kodoc(d->document);
+    // After setting the document (even if not changing it) we need to explicitly set the root area
+    // to 0. Otherwise crashes may occur when inserting textshape in words (or resetting document)
+    d->rootArea = 0;
+
+    if (d->document == document)
+        return;
+    d->document = document;
+
     if (kodoc.textEditor() == 0)
         kodoc.setTextEditor(new KoTextEditor(d->document));
 }
@@ -169,15 +183,6 @@ KoText::Direction KoTextShapeData::pageDirection() const
     return d->direction;
 }
 
-bool KoTextShapeData::isCursorVisible(QTextCursor *cursor) const
-{
-    Q_UNUSED(cursor);
-#ifdef __GNUC__
-    #warning FIXME: KoTextShapeData::isCursorVisible returns always true, why do we still need it?
-#endif
-    return true;
-}
-
 void KoTextShapeData::setRootArea(KoTextLayoutRootArea *rootArea)
 {
     Q_D(KoTextShapeData);
@@ -190,6 +195,62 @@ KoTextLayoutRootArea *KoTextShapeData::rootArea()
     return d->rootArea;
 }
 
+void KoTextShapeData::setLeftPadding(qreal padding)
+{
+    Q_D(KoTextShapeData);
+    d->leftPadding = padding;
+}
+
+qreal KoTextShapeData::leftPadding() const
+{
+    Q_D(const KoTextShapeData);
+    return d->leftPadding;
+}
+
+void KoTextShapeData::setTopPadding(qreal padding)
+{
+    Q_D(KoTextShapeData);
+    d->topPadding = padding;
+}
+
+qreal KoTextShapeData::topPadding() const
+{
+    Q_D(const KoTextShapeData);
+    return d->topPadding;
+}
+
+void KoTextShapeData::setRightPadding(qreal padding)
+{
+    Q_D(KoTextShapeData);
+    d->rightPadding = padding;
+}
+
+qreal KoTextShapeData::rightPadding() const
+{
+    Q_D(const KoTextShapeData);
+    return d->rightPadding;
+}
+
+void KoTextShapeData::setBottomPadding(qreal padding)
+{
+    Q_D(KoTextShapeData);
+    d->bottomPadding = padding;
+}
+
+qreal KoTextShapeData::bottomPadding() const
+{
+    Q_D(const KoTextShapeData);
+    return d->bottomPadding;
+}
+
+void KoTextShapeData::setPadding(qreal padding)
+{
+    setLeftPadding(padding);
+    setTopPadding(padding);
+    setRightPadding(padding);
+    setBottomPadding(padding);
+}
+
 bool KoTextShapeData::loadOdf(const KoXmlElement &element, KoShapeLoadingContext &context, KoDocumentRdfBase *rdfData, KoShape *shape)
 {
     Q_UNUSED(rdfData);
@@ -234,7 +295,33 @@ void KoTextShapeData::loadStyle(const KoXmlElement &element, KoShapeLoadingConte
         }
     }
 
+
     if (style) {
+        context.odfLoadingContext().addStyles(style, style->attributeNS(KoXmlNS::style, "family", "graphic").toLocal8Bit().constData());   // Load all parents
+        KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
+        styleStack.setTypeProperties("graphic");
+        // Spacing (padding)
+        const QString paddingLeft(styleStack.property(KoXmlNS::fo, "padding-left" ));
+        if (!paddingLeft.isEmpty()) {
+            setLeftPadding(KoUnit::parseValue(paddingLeft));
+        }
+        const QString paddingRight(styleStack.property(KoXmlNS::fo, "padding-right" ));
+        if (!paddingRight.isEmpty()) {
+            setRightPadding(KoUnit::parseValue(paddingRight));
+        }
+        const QString paddingTop(styleStack.property(KoXmlNS::fo, "padding-top" ));
+        if (!paddingTop.isEmpty()) {
+            setTopPadding(KoUnit::parseValue(paddingTop));
+        }
+        const QString paddingBottom(styleStack.property(KoXmlNS::fo, "padding-bottom" ));
+        if (!paddingBottom.isEmpty()) {
+            setBottomPadding(KoUnit::parseValue(paddingBottom));
+        }
+        const QString padding(styleStack.property(KoXmlNS::fo, "padding"));
+        if (!padding.isEmpty()) {
+            setPadding(KoUnit::parseValue(padding));
+        }
+
         // graphic styles don't support inheritance yet therefor some additional work is needed here.
         QList<KoParagraphStyle *> paragraphStyles;
         while (style) {
@@ -272,6 +359,23 @@ void KoTextShapeData::loadStyle(const KoXmlElement &element, KoShapeLoadingConte
 
 void KoTextShapeData::saveStyle(KoGenStyle &style, KoShapeSavingContext &context) const
 {
+    if ((leftPadding() == rightPadding()) && (topPadding() == bottomPadding()) && (rightPadding() == topPadding())) {
+        style.addPropertyPt("fo:padding", leftPadding(), KoGenStyle::GraphicType);
+    } else {
+        if (leftPadding()) {
+            style.addPropertyPt("fo:padding-left", leftPadding(), KoGenStyle::GraphicType);
+        }
+        if (rightPadding()) {
+            style.addPropertyPt("fo:padding-right", rightPadding(), KoGenStyle::GraphicType);
+        }
+        if (topPadding()) {
+            style.addPropertyPt("fo:padding-top", topPadding(), KoGenStyle::GraphicType);
+        }
+        if (bottomPadding()) {
+            style.addPropertyPt("fo:padding-bottom", bottomPadding(), KoGenStyle::GraphicType);
+        }
+    }
+
     Q_D(const KoTextShapeData);
     if (d->paragraphStyle) {
         d->paragraphStyle->saveOdf(style, context);
