@@ -1,7 +1,7 @@
 /*
  * KPlato Report Plugin
  * Copyright (C) 2007-2008 by Adam Pigg (adam@piggz.co.uk)
- * Copyright (C) 2010, 2011 by Dag Andersen <danders@get2net.dk>
+ * Copyright (C) 2010, 2011, 2012 by Dag Andersen <danders@get2net.dk>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -28,7 +28,8 @@
 #include "reportexportpanel.h"
 #include "ReportODTRenderer.h"
 #include "kptnodechartmodel.h"
-
+#include "kptdebug.h"
+#include "KoPageLayout.h"
 #include <KoReportPage.h>
 #include <KoReportPreRenderer.h>
 #include <KoReportPrintRenderer.h>
@@ -65,7 +66,8 @@
 #include <QPrinter>
 #include <QLabel>
 #include <QVBoxLayout>
-#include <QScrollArea>
+#include <QGraphicsView>
+#include <QGraphicsScene>
 #include <QLayout>
 #include <QDockWidget>
 #include <QModelIndex>
@@ -73,6 +75,7 @@
 #include <QStandardItemModel>
 #include <QActionGroup>
 
+
 namespace KPlato
 {
 
@@ -139,15 +142,19 @@ QAbstractPrintDialog::PrintDialogOptions ReportPrintingDialog::printDialogOption
 ReportView::ReportView( KoDocument *part, QWidget *parent )
     : ViewBase( part, parent )
 {
-//    kDebug()<<"--------------- ReportView ------------------";
+//    kDebug(planDbg())<<"--------------- ReportView ------------------";
 
     m_preRenderer = 0;
     setObjectName("ReportView");
-    m_scrollArea = new QScrollArea(this);
-    m_scrollArea->setBackgroundRole(QPalette::Dark);
-    m_scrollArea->viewport()->setAutoFillBackground(true);
+    
+    m_reportView = new QGraphicsView(this);   
+    m_reportScene = new QGraphicsScene(this);
+    m_reportScene->setSceneRect(0,0,1000,2000);
+    m_reportView->setScene(m_reportScene);
+    m_reportScene->setBackgroundBrush(palette().brush(QPalette::Dark));
+    
     QVBoxLayout *l = new QVBoxLayout( this );
-    l->addWidget( m_scrollArea );
+    l->addWidget( m_reportView );
     m_pageSelector = new ReportNavigator( this );
     l->addWidget( m_pageSelector );
 
@@ -267,7 +274,7 @@ QMap<QString, QAbstractItemModel*> ReportView::createReportModels( Project *proj
 
 void ReportView::renderPage( int page )
 {
-    m_reportWidget->renderPage( page );
+    m_reportPage->renderPage( page );
 }
 
 void ReportView::nextPage()
@@ -318,6 +325,29 @@ void ReportView::slotExport()
     dia->activateWindow();
 }
 
+KoPageLayout ReportView::pageLayout() const
+{
+    KoPageLayout p = ViewBase::pageLayout();
+    ReportPageOptions opt = m_reportDocument->pageOptions();
+    p.orientation = opt.isPortrait() ? KoPageFormat::Portrait : KoPageFormat::Landscape;
+
+    if (opt.getPageSize().isEmpty()) {
+        p.format = KoPageFormat::CustomSize;
+        p.width = opt.getCustomWidth();
+        p.height = opt.getCustomHeight();
+    } else {
+        p.format = KoPageFormat::formatFromString(opt.getPageSize());
+    }
+    p.topMargin = opt.getMarginTop();
+    p.bottomMargin = opt.getMarginBottom();
+    p.leftMargin = opt.getMarginLeft();
+    p.rightMargin = opt.getMarginRight();
+
+    p.pageEdge = 0.0;
+    p.bindingSide = 0.0;
+    return p;
+}
+
 void ReportView::slotExportFinished( int result )
 {
     //TODO confirm overwrite
@@ -349,7 +379,7 @@ void ReportView::slotExportFinished( int result )
 
 void ReportView::exportToOdt( KoReportRendererContext &context )
 {
-    kDebug()<<"Export to odt:"<<context.destinationUrl;
+    kDebug(planDbg())<<"Export to odt:"<<context.destinationUrl;
     KoReportRendererBase *renderer = new ReportODTRenderer();
 //    renderer = m_factory.createInstance("odt");
     if ( renderer == 0 ) {
@@ -363,7 +393,7 @@ void ReportView::exportToOdt( KoReportRendererContext &context )
 
 void ReportView::exportToOds( KoReportRendererContext &context )
 {
-    kDebug()<<"Export to ods:"<<context.destinationUrl;
+    kDebug(planDbg())<<"Export to ods:"<<context.destinationUrl;
     KoReportRendererBase *renderer;
     renderer = m_factory.createInstance("ods");
     if ( renderer == 0 ) {
@@ -377,7 +407,7 @@ void ReportView::exportToOds( KoReportRendererContext &context )
 
 void ReportView::exportToHtml( KoReportRendererContext &context )
 {
-    kDebug()<<"Export to html:"<<context.destinationUrl;
+    kDebug(planDbg())<<"Export to html:"<<context.destinationUrl;
     KoReportRendererBase *renderer;
     renderer = m_factory.createInstance("htmltable");
     if ( renderer == 0 ) {
@@ -391,7 +421,7 @@ void ReportView::exportToHtml( KoReportRendererContext &context )
 
 void ReportView::exportToXHtml( KoReportRendererContext &context )
 {
-    kDebug()<<"Export to xhtml:"<<context.destinationUrl;
+    kDebug(planDbg())<<"Export to xhtml:"<<context.destinationUrl;
     KoReportRendererBase *renderer;
     renderer = m_factory.createInstance("htmlcss");
     if ( renderer == 0 ) {
@@ -444,14 +474,14 @@ void ReportView::setReportModels( const QMap<QString, QAbstractItemModel*> &map
 void ReportView::slotRefreshView()
 {
     if ( ! isVisible() ) {
-        kDebug()<<"Not visible";
+        kDebug(planDbg())<<"Not visible";
         return;
     }
     delete m_preRenderer;
     QDomElement e = m_design.documentElement();
     m_preRenderer = new KoReportPreRenderer( e.firstChildElement( "report:content" ) );
     if ( ! m_preRenderer->isValid()) {
-        kDebug()<<"Invalid design document";
+        kDebug(planDbg())<<"Invalid design document";
         return;
     }
     ReportData *rd = createReportData( e );
@@ -462,9 +492,14 @@ void ReportView::slotRefreshView()
     m_pageSelector->setMaximum( m_reportDocument ? m_reportDocument->pages() : 1 );
     m_pageSelector->setCurrentPage( 1 );
 
-    m_reportWidget = new KoReportPage(this, m_reportDocument);
-    m_reportWidget->setObjectName("ReportPage");
-    m_scrollArea->setWidget(m_reportWidget);
+    m_reportPage = new KoReportPage(this, m_reportDocument);
+    m_reportPage->setObjectName("ReportPage");
+    
+    m_reportScene->setSceneRect(0,0,m_reportPage->rect().width() + 40, m_reportPage->rect().height() + 40);
+    m_reportScene->addItem(m_reportPage);
+    m_reportPage->setPos(20,20);
+    m_reportView->centerOn(0,0);
+	    
     return;
 }
 
@@ -485,7 +520,7 @@ void ReportView::createReportData( const QString &type, ReportData *rd )
 
 ReportData *ReportView::createReportData( const QString &type )
 {
-    kDebug()<<type;
+    kDebug(planDbg())<<type;
     //FIXME a smarter report data creator
     ReportData *r = 0;
     if ( type == "costbreakdown" || type =="costperformance" || type =="effortperformance" ) {
@@ -493,6 +528,9 @@ ReportData *ReportView::createReportData( const QString &type )
         static_cast<ChartReportData*>( r )->cbs = ( type == "costbreakdown" ? true : false );
     } else {
         r = new ReportData();
+        if ( type == "tasks" || type == "taskstatus" ) {
+            r->setColumnRole( NodeModel::NodeDescription, Qt::EditRole );
+        }
     }
     r->setModel( m_modelmap.value( type ) );
     r->setProject( project() );
@@ -512,16 +550,15 @@ bool ReportView::loadXML( const QDomDocument &doc )
 
 bool ReportView::loadContext( const KoXmlElement &context )
 {
-    QDomDocument doc( "context" );
-    QDomElement e = KoXml::asQDomElement( doc, context ).firstChildElement( "planreportdefinition" );
+    KoXmlElement e = context.namedItem( "planreportdefinition" ).toElement();
     if ( e.isNull() ) {
-        // try to handle old definition
-        e = KoXml::asQDomElement( doc, context ).firstChildElement( "kplatoreportdefinition" );
+         e = context.namedItem( "kplatoreportdefinition" ).toElement();
     }
     if ( ! e.isNull() ) {
-        m_design = QDomDocument( "context" );
-        m_design.appendChild( e );
-    } else kDebug()<<"Invalid context xml";
+        QDomDocument doc( "context" );
+        KoXml::asQDomElement( doc, e );
+        m_design = doc;
+    } else kDebug(planDbg())<<"Invalid context xml";
     slotRefreshView();
     return true;
 }
@@ -672,8 +709,11 @@ void ReportDesignDialog::slotButtonClicked( int button )
 
 void ReportDesignDialog::slotSaveToFile()
 {
-    KFileDialog *dialog = new KFileDialog(KUrl(), QString(), this);
+    QPointer<KFileDialog> dialog = new KFileDialog(KUrl(), QString(), this);
     dialog->exec();
+    if ( ! dialog ) {
+        return;
+    }
     KUrl url(dialog->selectedUrl());
     delete dialog;
 
@@ -852,6 +892,7 @@ void ReportDesignPanel::populateToolbar( KToolBar *tb )
         << "report:check"
         << "report:line"
         << "report:chart"
+        << "report:web"
         << ""; //separator
     foreach( QAction *a, m_designer->actions(m_actionGroup) ) {
         if ( ! itemtypes.contains( a->objectName() ) ) {
@@ -904,8 +945,8 @@ QDomDocument ReportDesignPanel::document() const
         m_sourceeditor->sourceData( e );
     }
     e.appendChild( m_designer->document() );
-/*    qDebug()<<"ReportDesignerView::document:";
-    qDebug()<<document.toString();*/
+/*    kDebug(planDbg())<<"ReportDesignerView::document:";
+    kDebug(planDbg())<<document.toString();*/
     return document;
 }
 
