@@ -42,7 +42,7 @@
 #include <KoGenStyles.h>
 #include <KoUnit.h>
 #include <KoColorBackground.h>
-#include <KoLineBorder.h>
+#include <KoShapeStroke.h>
 
 // KDChart
 #include <KDChartChart>
@@ -79,7 +79,7 @@ public:
     QFont titleFont;
     QColor fontColor;
     Qt::Alignment alignment;
-    KoLineBorder *lineBorder;
+    KoShapeStroke *lineBorder;
 
     // The connection to KDChart
     KDChart::Legend *kdLegend;
@@ -94,7 +94,7 @@ public:
 
 Legend::Private::Private()
 {
-    lineBorder = new KoLineBorder( 0.5, Qt::black );
+    lineBorder = new KoShapeStroke(0.5, Qt::black);
     showFrame = true;
     framePen = QPen();
     backgroundBrush = QBrush();
@@ -110,31 +110,31 @@ Legend::Private::~Private()
 }
 
 
-Legend::Legend( ChartShape *parent )
-    : QObject( parent )
-    , d( new Private() )
+Legend::Legend(ChartShape *parent)
+    : QObject(parent)
+    , d(new Private())
 {
-    Q_ASSERT( parent );
+    Q_ASSERT(parent);
 
-    setShapeId( ChartShapeId );
+    setShapeId(ChartShapeId);
 
     d->shape = parent;
 
     d->kdLegend = new KDChart::Legend();
-    d->kdLegend->setTextAlignment( Qt::AlignLeft | Qt::AlignVCenter );
+    d->kdLegend->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
 
-    setTitleFontSize( 10 );
-    setTitle( QString() );
-    setFontSize( 8 );
+    setTitleFontSize(10);
+    setTitle(QString());
+    setFontSize(8);
 
     update();
 
-    parent->addShape( this );
+    parent->addShape(this);
 
-    connect ( d->kdLegend, SIGNAL( propertiesChanged() ),
-              this,        SLOT( slotKdLegendChanged() ) );
-    connect ( parent, SIGNAL( chartTypeChanged( ChartType ) ),
-              this,   SLOT( slotChartTypeChanged( ChartType ) ) );
+    connect (d->kdLegend, SIGNAL(propertiesChanged()),
+             this,        SLOT(slotKdLegendChanged()));
+    connect (parent, SIGNAL(chartTypeChanged(ChartType)),
+             this,   SLOT(slotChartTypeChanged(ChartType)));
 }
 
 Legend::~Legend()
@@ -149,10 +149,10 @@ QString Legend::title() const
     return d->title;
 }
 
-void Legend::setTitle( const QString &title )
+void Legend::setTitle(const QString &title)
 {
     d->title = title;
-    d->kdLegend->setTitleText( title );
+    d->kdLegend->setTitleText(title);
     d->pixmapRepaintRequested = true;
 }
 
@@ -161,10 +161,10 @@ bool Legend::showFrame() const
     return d->showFrame;
 }
 
-void Legend::setShowFrame( bool show )
+void Legend::setShowFrame(bool show)
 {
     d->showFrame = show;
-    setBorder( show ? d->lineBorder : 0 );
+    setStroke(show ? d->lineBorder : 0);
 }
 
 QPen Legend::framePen() const
@@ -172,14 +172,14 @@ QPen Legend::framePen() const
     return d->framePen;
 }
 
-void Legend::setFramePen( const QPen &pen )
+void Legend::setFramePen(const QPen &pen)
 {
     d->framePen = pen;
 
     // KDChart
     KDChart::FrameAttributes attributes = d->kdLegend->frameAttributes();
-    attributes.setPen( pen  );
-    d->kdLegend->setFrameAttributes( attributes );
+    attributes.setPen(pen);
+    d->kdLegend->setFrameAttributes(attributes);
 
     d->pixmapRepaintRequested = true;
 }
@@ -189,17 +189,17 @@ QColor Legend::frameColor() const
     return d->framePen.color();
 }
 
-void Legend::setFrameColor( const QColor &color )
+void Legend::setFrameColor(const QColor &color)
 {
-    d->framePen.setColor( color );
+    d->framePen.setColor(color);
 
     // KDChart
     KDChart::FrameAttributes attributes = d->kdLegend->frameAttributes();
-    attributes.setVisible( true );
+    attributes.setVisible(true);
     QPen pen = attributes.pen();
-    pen.setColor( color );
-    attributes.setPen( pen );
-    d->kdLegend->setFrameAttributes( attributes );
+    pen.setColor(color);
+    attributes.setPen(pen);
+    d->kdLegend->setFrameAttributes(attributes);
 
     d->pixmapRepaintRequested = true;
 }
@@ -209,15 +209,15 @@ QBrush Legend::backgroundBrush() const
     return d->backgroundBrush;
 }
 
-void Legend::setBackgroundBrush( const QBrush &brush )
+void Legend::setBackgroundBrush(const QBrush &brush)
 {
     d->backgroundBrush = brush;
 
     // KDChart
     KDChart::BackgroundAttributes attributes = d->kdLegend->backgroundAttributes();
-    attributes.setVisible( true );
-    attributes.setBrush( brush );
-    d->kdLegend->setBackgroundAttributes( attributes );
+    attributes.setVisible(true);
+    attributes.setBrush(brush);
+    d->kdLegend->setBackgroundAttributes(attributes);
 
     d->pixmapRepaintRequested = true;
 }
@@ -227,17 +227,17 @@ QColor Legend::backgroundColor() const
     return d->backgroundBrush.color();
 }
 
-void Legend::setBackgroundColor( const QColor &color )
+void Legend::setBackgroundColor(const QColor &color)
 {
-    d->backgroundBrush.setColor( color );
+    d->backgroundBrush.setColor(color);
 
     // KDChart
     KDChart::BackgroundAttributes attributes = d->kdLegend->backgroundAttributes();
-    attributes.setVisible( true );
+    attributes.setVisible(true);
     QBrush brush = attributes.brush();
-    brush.setColor( color );
-    attributes.setBrush( brush );
-    d->kdLegend->setBackgroundAttributes( attributes );
+    brush.setColor(color);
+    attributes.setBrush(brush);
+    d->kdLegend->setBackgroundAttributes(attributes);
 
     d->pixmapRepaintRequested = true;
 }
@@ -247,14 +247,14 @@ QFont Legend::font() const
     return d->font;
 }
 
-void Legend::setFont( const QFont &font )
+void Legend::setFont(const QFont &font)
 {
     d->font = font;
 
     // KDChart
     KDChart::TextAttributes attributes = d->kdLegend->textAttributes();
-    attributes.setFont( font );
-    d->kdLegend->setTextAttributes( attributes );
+    attributes.setFont(font);
+    d->kdLegend->setTextAttributes(attributes);
 
     d->pixmapRepaintRequested = true;
 }
@@ -264,27 +264,27 @@ qreal Legend::fontSize() const
     return d->font.pointSizeF();
 }
 
-void Legend::setFontSize( qreal size )
+void Legend::setFontSize(qreal size)
 {
-    d->font.setPointSizeF( size );
+    d->font.setPointSizeF(size);
 
     // KDChart
     KDChart::TextAttributes attributes = d->kdLegend->textAttributes();
     KDChart::Measure m = attributes.fontSize();
-    m.setValue( size );
-    attributes.setFontSize( m );
-    d->kdLegend->setTextAttributes( attributes );
+    m.setValue(size);
+    attributes.setFontSize(m);
+    d->kdLegend->setTextAttributes(attributes);
 
     d->pixmapRepaintRequested = true;
 }
 
-void Legend::setFontColor( const QColor &color )
+void Legend::setFontColor(const QColor &color)
 {
     KDChart::TextAttributes attributes = d->kdLegend->textAttributes();
     QPen pen = attributes.pen();
     pen.setColor(color);
     attributes.setPen(pen);
-    d->kdLegend->setTextAttributes( attributes );
+    d->kdLegend->setTextAttributes(attributes);
 
     d->pixmapRepaintRequested = true;
 }
@@ -301,14 +301,14 @@ QFont Legend::titleFont() const
     return d->titleFont;
 }
 
-void Legend::setTitleFont( const QFont &font )
+void Legend::setTitleFont(const QFont &font)
 {
     d->titleFont = font;
 
     // KDChart
     KDChart::TextAttributes attributes = d->kdLegend->titleTextAttributes();
-    attributes.setFont( font );
-    d->kdLegend->setTitleTextAttributes( attributes );
+    attributes.setFont(font);
+    d->kdLegend->setTitleTextAttributes(attributes);
 
     d->pixmapRepaintRequested = true;
 }
@@ -318,14 +318,14 @@ qreal Legend::titleFontSize() const
     return d->titleFont.pointSizeF();
 }
 
-void Legend::setTitleFontSize( qreal size )
+void Legend::setTitleFontSize(qreal size)
 {
-    d->titleFont.setPointSizeF( size );
+    d->titleFont.setPointSizeF(size);
 
     // KDChart
     KDChart::TextAttributes attributes = d->kdLegend->titleTextAttributes();
-    attributes.setFontSize( KDChart::Measure( size, KDChartEnums::MeasureCalculationModeAbsolute ) );
-    d->kdLegend->setTitleTextAttributes( attributes );
+    attributes.setFontSize(KDChart::Measure(size, KDChartEnums::MeasureCalculationModeAbsolute));
+    d->kdLegend->setTitleTextAttributes(attributes);
 
     d->pixmapRepaintRequested = true;
 }
@@ -335,10 +335,10 @@ LegendExpansion Legend::expansion() const
     return d->expansion;
 }
 
-void Legend::setExpansion( LegendExpansion expansion )
+void Legend::setExpansion(LegendExpansion expansion)
 {
     d->expansion = expansion;
-    d->kdLegend->setOrientation( LegendExpansionToQtOrientation( expansion ) );
+    d->kdLegend->setOrientation(LegendExpansionToQtOrientation(expansion));
     d->pixmapRepaintRequested = true;
 }
 
@@ -347,7 +347,7 @@ Qt::Alignment Legend::alignment() const
     return d->alignment;
 }
 
-void Legend::setAlignment( Qt::Alignment alignment )
+void Legend::setAlignment(Qt::Alignment alignment)
 {
     d->alignment = alignment;
 }
@@ -357,58 +357,58 @@ Position Legend::legendPosition() const
     return d->position;
 }
 
-void Legend::setLegendPosition( Position position )
+void Legend::setLegendPosition(Position position)
 {
     d->position = position;
     d->pixmapRepaintRequested = true;
 
-    d->shape->layout()->setPosition( this, position );
+    d->shape->layout()->setPosition(this, position);
 }
 
-void Legend::setSize( const QSizeF &newSize )
+void Legend::setSize(const QSizeF &newSize)
 {
-    QSize newSizePx = ScreenConversions::scaleFromPtToPx( newSize );
-    d->kdLegend->resize( newSizePx );
-    d->kdLegend->resizeLayout( newSizePx );
-    KoShape::setSize( newSize );
+    QSize newSizePx = ScreenConversions::scaleFromPtToPx(newSize);
+    d->kdLegend->resize(newSizePx);
+    d->kdLegend->resizeLayout(newSizePx);
+    KoShape::setSize(newSize);
 }
 
 
-void Legend::paintPixmap( QPainter &painter, const KoViewConverter &converter )
+void Legend::paintPixmap(QPainter &painter, const KoViewConverter &converter)
 {
     // Adjust the size of the painting area to the current zoom level
-    const QSize paintRectSize = converter.documentToView( d->lastSize ).toSize();
-    d->image = QImage( paintRectSize, QImage::Format_ARGB32 );
+    const QSize paintRectSize = converter.documentToView(d->lastSize).toSize();
+    d->image = QImage(paintRectSize, QImage::Format_ARGB32);
 
-    QPainter pixmapPainter( &d->image );
-    pixmapPainter.setRenderHints( painter.renderHints() );
-    pixmapPainter.setRenderHint( QPainter::Antialiasing, false );
+    QPainter pixmapPainter(&d->image);
+    pixmapPainter.setRenderHints(painter.renderHints());
+    pixmapPainter.setRenderHint(QPainter::Antialiasing, false);
 
     // Scale the painter's coordinate system to fit the current zoom level.
-    applyConversion( pixmapPainter, converter );
-    d->kdLegend->paint( &pixmapPainter );
+    applyConversion(pixmapPainter, converter);
+    d->kdLegend->paint(&pixmapPainter);
 }
 
-void Legend::paint( QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &)
+void Legend::paint(QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &)
 {
     //painter.save();
 
     // First of all, scale the painter's coordinate system to fit the current zoom level
-    applyConversion( painter, converter );
+    applyConversion(painter, converter);
 
     // Calculate the clipping rect
-    QRectF paintRect = QRectF( QPointF( 0, 0 ), size() );
-    //clipRect.intersect( paintRect );
-    painter.setClipRect( paintRect, Qt::IntersectClip );
+    QRectF paintRect = QRectF(QPointF(0, 0), size());
+    //clipRect.intersect(paintRect);
+    painter.setClipRect(paintRect, Qt::IntersectClip);
 
     // Get the current zoom level
     QPointF zoomLevel;
-    converter.zoom( &zoomLevel.rx(), &zoomLevel.ry() );
+    converter.zoom(&zoomLevel.rx(), &zoomLevel.ry());
 
     // Only repaint the pixmap if it is scheduled, the zoom level changed or the shape was resized
-    /*if (    d->pixmapRepaintRequested
+    /*if (   d->pixmapRepaintRequested
          || d->lastZoomLevel != zoomLevel
-         || d->lastSize      != size() ) {
+         || d->lastSize      != size()) {
         // TODO: What if two zoom levels are constantly being requested?
         // At the moment, this *is* the case, due to the fact
         // that the shape is also rendered in the page overview
@@ -420,117 +420,130 @@ void Legend::paint( QPainter &painter, const KoViewConverter &converter, KoShape
         d->lastZoomLevel = zoomLevel;
         d->lastSize      = size();
 
-        paintPixmap( painter, converter );
+        paintPixmap(painter, converter);
     }*/
 
     // Paint the background
-    if ( background() ) {
+    if (background()) {
         QPainterPath p;
-        p.addRect( paintRect );
-        background()->paint( painter, p );
+        p.addRect(paintRect);
+        background()->paint(painter, p);
     }
 
     // KDChart thinks in pixels, Calligra in pt
-    ScreenConversions::scaleFromPtToPx( painter );
+    ScreenConversions::scaleFromPtToPx(painter);
 
-    d->kdLegend->paint( &painter );
+    d->kdLegend->paint(&painter);
 
     //painter.restore();
     // Paint the cached pixmap
-    //painter.drawImage( 0, 0, d->image );
+    //painter.drawImage(0, 0, d->image);
 }
 
 
-// Only reimplemented because pure virtual in KoShape, but not needed
-bool Legend::loadOdf( const KoXmlElement &legendElement,
-                      KoShapeLoadingContext &context )
+// ----------------------------------------------------------------
+//                     loading and saving
+
+
+bool Legend::loadOdf(const KoXmlElement &legendElement,
+                     KoShapeLoadingContext &context)
 {
     KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
     styleStack.clear();
     
     // FIXME: If the style isn't present we shouldn't care about it at all
     // and move everything related to the legend style in this if clause
-    if ( legendElement.hasAttributeNS( KoXmlNS::chart, "style-name" ) ) {
-        context.odfLoadingContext().fillStyleStack( legendElement, KoXmlNS::chart, "style-name", "chart" );
-        styleStack.setTypeProperties( "graphic" );
+    if (legendElement.hasAttributeNS(KoXmlNS::chart, "style-name")) {
+        context.odfLoadingContext().fillStyleStack(legendElement, KoXmlNS::chart, "style-name", "chart");
+        styleStack.setTypeProperties("graphic");
     }
 
-    if ( !legendElement.isNull() ) {
+    if (!legendElement.isNull()) {
         int attributesToLoad = OdfAllAttributes;
-        QString lp = legendElement.attributeNS( KoXmlNS::chart, "legend-position", QString() );
+        QString lp = legendElement.attributeNS(KoXmlNS::chart, "legend-position", QString());
         if (!lp.isEmpty()) {
             attributesToLoad ^= OdfPosition;
         }
 
-        // FIXME according to odf if legend-position is provided the x and y value should not be used
-        // FIXME also with and height are not supported at this place
-        if ( legendElement.hasAttributeNS( KoXmlNS::svg, "x" ) ||
-             legendElement.hasAttributeNS( KoXmlNS::svg, "y" ) ||
-             legendElement.hasAttributeNS( KoXmlNS::svg, "width" ) ||
-             legendElement.hasAttributeNS( KoXmlNS::svg, "height" ) )
-            d->shape->layout()->setPosition( this, FloatingPosition );
+        // FIXME according to odf if legend-position is provided the x
+        // and y value should not be used.
+        //
+        // FIXME also width and height are not supported at this place
+        if (legendElement.hasAttributeNS(KoXmlNS::svg, "x") ||
+            legendElement.hasAttributeNS(KoXmlNS::svg, "y") ||
+            legendElement.hasAttributeNS(KoXmlNS::svg, "width") ||
+            legendElement.hasAttributeNS(KoXmlNS::svg, "height"))
+        {
+            d->shape->layout()->setPosition(this, FloatingPosition);
+        }
 
-        loadOdfAttributes( legendElement, context, attributesToLoad );
+        loadOdfAttributes(legendElement, context, attributesToLoad);
 
-        QString lalign = legendElement.attributeNS( KoXmlNS::chart, "legend-align", QString() );
+        QString lalign = legendElement.attributeNS(KoXmlNS::chart, "legend-align", QString());
 
-        if ( legendElement.hasAttributeNS( KoXmlNS::style, "legend-expansion" ) ) {
-            QString lexpansion = legendElement.attributeNS( KoXmlNS::style, "legend-expansion", QString() );
-            if ( lexpansion == "wide" )
-                setExpansion( WideLegendExpansion );
-            else if ( lexpansion == "high" )
-                setExpansion( HighLegendExpansion );
+        if (legendElement.hasAttributeNS(KoXmlNS::style, "legend-expansion")) {
+            QString lexpansion = legendElement.attributeNS(KoXmlNS::style, "legend-expansion", QString());
+            if (lexpansion == "wide")
+                setExpansion(WideLegendExpansion);
+            else if (lexpansion == "high")
+                setExpansion(HighLegendExpansion);
             else
-                setExpansion( BalancedLegendExpansion );
+                setExpansion(BalancedLegendExpansion);
         }
 
-        if ( lalign == "start" ) {
-            setAlignment( Qt::AlignLeft );
+        if (lalign == "start") {
+            setAlignment(Qt::AlignLeft);
         }
-        else if ( lalign == "end" ) {
-            setAlignment( Qt::AlignRight );
+        else if (lalign == "end") {
+            setAlignment(Qt::AlignRight);
         }
         else {
-            setAlignment( Qt::AlignCenter );
+            setAlignment(Qt::AlignCenter);
         }
 
-        if ( lp == "start" ) {
-            setLegendPosition( StartPosition );
+        if (lp == "start") {
+            setLegendPosition(StartPosition);
         }
-        else if ( lp == "top" ) {
-            setLegendPosition( TopPosition );
+        else if (lp == "top") {
+            setLegendPosition(TopPosition);
         }
-        else if ( lp == "bottom" ) {
-            setLegendPosition( BottomPosition );
+        else if (lp == "bottom") {
+            setLegendPosition(BottomPosition);
         }
-        else if ( lp == "top-start" ) {
-            setLegendPosition( TopStartPosition );
+        else if (lp == "top-start") {
+            setLegendPosition(TopStartPosition);
         }
-        else if ( lp == "bottom-start" ) {
-            setLegendPosition( BottomStartPosition );
+        else if (lp == "bottom-start") {
+            setLegendPosition(BottomStartPosition);
         }
-        else if ( lp == "top-end" ) {
-            setLegendPosition( TopEndPosition );
+        else if (lp == "top-end") {
+            setLegendPosition(TopEndPosition);
         }
-        else if ( lp == "bottom-end" ) {
-            setLegendPosition( BottomEndPosition );
+        else if (lp == "bottom-end") {
+            setLegendPosition(BottomEndPosition);
         }
         else {
-            setLegendPosition( EndPosition );
+            setLegendPosition(EndPosition);
         }
 
-        if ( legendElement.hasAttributeNS( KoXmlNS::calligra, "title" ) ) {
-            setTitle( legendElement.attributeNS( KoXmlNS::calligra,
-                                                       "title", QString() ) );
+        if (legendElement.hasAttributeNS(KoXmlNS::office, "title")) {
+            setTitle(legendElement.attributeNS(KoXmlNS::office, "title", QString()));
         }
 
-        styleStack.setTypeProperties( "text" );
+        styleStack.setTypeProperties("text");
 
-        if ( styleStack.hasProperty( KoXmlNS::fo, "font-size" ) ) {
-            setFontSize( KoUnit::parseValue( styleStack.property( KoXmlNS::fo, "font-size" ) ) );
+        if (styleStack.hasProperty(KoXmlNS::fo, "font-family")) {
+            QString fontFamily = styleStack.property(KoXmlNS::fo, "font-family");
+            QFont font = d->font;
+            font.setFamily(fontFamily);
+            setFont(font);
         }
-        if ( styleStack.hasProperty( KoXmlNS::fo, "font-color" ) ) {
-            QColor color = styleStack.property( KoXmlNS::fo, "font-color" );
+        if (styleStack.hasProperty(KoXmlNS::fo, "font-size")) {
+            qreal fontSize = KoUnit::parseValue(styleStack.property(KoXmlNS::fo, "font-size"));
+            setFontSize(fontSize);
+        }
+        if (styleStack.hasProperty(KoXmlNS::fo, "font-color")) {
+            QColor color = styleStack.property(KoXmlNS::fo, "font-color");
             if (color.isValid()) {
                 setFontColor(color);
             }
@@ -539,65 +552,55 @@ bool Legend::loadOdf( const KoXmlElement &legendElement,
     else {
         // No legend element, use default legend.
         // FIXME: North??  Isn't that a bit strange as default? /IW
-        setLegendPosition( TopPosition );
-        setAlignment( Qt::AlignCenter );
+        setLegendPosition(TopPosition);
+        setAlignment(Qt::AlignCenter);
     }
 
-    //d->chart->replaceLegend( d->legend, oldLegend );
-
     d->pixmapRepaintRequested = true;
 
     return true;
 }
 
-void Legend::saveOdf( KoShapeSavingContext &context ) const
+void Legend::saveOdf(KoShapeSavingContext &context) const
 {
     KoXmlWriter &bodyWriter = context.xmlWriter();
-    KoGenStyles &mainStyles = context.mainStyles();
 
-    bodyWriter.startElement( "chart:legend" );
+    bodyWriter.startElement("chart:legend");
+    saveOdfAttributes(context, OdfPosition);
 
-    saveOdfAttributes( context, OdfPosition );
-
-    QString lp = PositionToString( d->position );
-
-    QString lalign;
-
-    if ( !lp.isEmpty() ) {
-        bodyWriter.addAttribute( "chart:legend-position", lp );
+    // Legend specific attributes
+    QString lp = PositionToString(d->position);
+    if (!lp.isEmpty()) {
+        bodyWriter.addAttribute("chart:legend-position", lp);
     }
-    if ( !lalign.isEmpty() ) {
-        bodyWriter.addAttribute( "chart:legend-align", lalign );
+    QString lalign;  // FIXME: This string is always empty.  What gives?
+    if (!lalign.isEmpty()) {
+        bodyWriter.addAttribute("chart:legend-align", lalign);
     }
 
+    // Legend style FIXME: Check if more styling then just the font goes here.
     KoGenStyle style(KoGenStyle::ChartAutoStyle, "chart", 0);
     saveOdfFont(style, d->font, d->fontColor);
-
-    bodyWriter.addAttribute( "chart:style-name", saveStyle(style, context) );
+    bodyWriter.addAttribute("chart:style-name", saveStyle(style, context));
 
     QString  lexpansion;
-    switch ( expansion() ) {
-    case WideLegendExpansion:
-        lexpansion = "wide";
-        break;
-    case HighLegendExpansion:
-        lexpansion = "high";
-        break;
-    case BalancedLegendExpansion:
-        lexpansion = "balanced";
-        break;
+    switch (expansion()) {
+    case WideLegendExpansion:      lexpansion = "wide";      break;
+    case HighLegendExpansion:      lexpansion = "high";      break;
+    case BalancedLegendExpansion:  lexpansion = "balanced";  break;
     };
+    bodyWriter.addAttribute("style:legend-expansion", lexpansion);
+
+    if (!title().isEmpty())
+        bodyWriter.addAttribute("office:title", title());
 
-    bodyWriter.addAttribute( "style:legend-expansion", lexpansion );
-    if ( !title().isEmpty() )
-        bodyWriter.addAttribute( "office:title", title() );
     bodyWriter.endElement(); // chart:legend
 }
 
 KDChart::Legend *Legend::kdLegend() const
 {
     // There has to be a valid KDChart instance of this legend
-    Q_ASSERT( d->kdLegend );
+    Q_ASSERT(d->kdLegend);
     return d->kdLegend;
 }
 
@@ -619,21 +622,21 @@ void Legend::slotKdLegendChanged()
     // in KDChartModel. Right now, only yDataChanged() is implemented.
     //d->kdLegend->forceRebuild();
     QSize size = d->kdLegend->sizeHint();
-    setSize( ScreenConversions::scaleFromPxToPt( size ) );
+    setSize(ScreenConversions::scaleFromPxToPt(size));
     update();
 }
 
-void Legend::slotChartTypeChanged( ChartType chartType )
+void Legend::slotChartTypeChanged(ChartType chartType)
 {
     // TODO: Once we support markers, this switch will have to be
     // more clever.
-    switch ( chartType ) {
+    switch (chartType) {
     case LineChartType:
     //case ScatterChartType:
-        d->kdLegend->setLegendStyle( KDChart::Legend::LinesOnly );
+        d->kdLegend->setLegendStyle(KDChart::Legend::LinesOnly);
         break;
     default:
-        d->kdLegend->setLegendStyle( KDChart::Legend::MarkersOnly );
+        d->kdLegend->setLegendStyle(KDChart::Legend::MarkersOnly);
         break;
     }
 }
