@@ -167,6 +167,7 @@ void TextShape::paintComponent(QPainter &painter, const KoViewConverter &convert
 
     painter.save();
     painter.translate(0, -m_textShapeData->documentOffset());
+    painter.translate(m_textShapeData->leftPadding(), m_textShapeData->topPadding());
     m_textShapeData->rootArea()->paint(&painter, pc); // only need to draw ourselves
     painter.restore();
 
@@ -407,3 +408,8 @@ void TextShape::waitUntilReady(const KoViewConverter &, bool asynchronous) const
         lay->layout();
     }
 }
+
+KoImageCollection *TextShape::imageCollection()
+{
+    return m_imageCollection;
+}
