@@ -59,7 +59,6 @@
 
 // ui
 #include "ui/CellEditor.h"
-#include "ui/CellToolOptionWidget.h"
 #include "ui/ExternalEditor.h"
 #include "ui/SheetView.h"
 
@@ -87,11 +86,11 @@ void CellToolBase::Private::updateEditor(const Cell& cell)
     const Cell& theCell = cell.isPartOfMerged() ? cell.masterCell() : cell;
     const Style style = theCell.style();
     if (q->selection()->activeSheet()->isProtected() && style.hideFormula()) {
-        optionWidget->editor()->setPlainText(theCell.displayText());
+        externalEditor->setPlainText(theCell.displayText());
     } else if (q->selection()->activeSheet()->isProtected() && style.hideAll()) {
-        optionWidget->editor()->clear();
+        externalEditor->clear();
     } else {
-        optionWidget->editor()->setPlainText(theCell.userInput());
+        externalEditor->setPlainText(theCell.userInput());
     }
 }
 
@@ -183,8 +182,8 @@ void CellToolBase::Private::setProtectedActionsEnabled(bool enable)
     const QList<KAction*> actions = q->actions().values();
     for (int i = 0; i < actions.count(); ++i)
         actions[i]->setEnabled(enable);
-    optionWidget->formulaButton()->setEnabled(enable);
-    optionWidget->editor()->setEnabled(enable);
+    q->action("insertFormula")->setEnabled(enable);
+    externalEditor->setEnabled(enable);
 
     // These actions are always enabled.
     q->action("copy")->setEnabled(true);
@@ -1234,9 +1233,9 @@ QList<QAction*> CellToolBase::Private::popupActionList() const
             }
         }
         actions.append(popupMenuActions["separator6"]);
-        actions.append(q->action("comment"));
+        actions.append(popupMenuActions["comment"]);
         if (!cell.comment().isEmpty()) {
-            actions.append(q->action("clearComment"));
+            actions.append(popupMenuActions["clearComment"]);
         }
 
         if (testListChoose(q->selection())) {
@@ -1269,11 +1268,11 @@ void CellToolBase::Private::createPopupMenuActions()
     connect(action, SIGNAL(triggered(bool)), q, SLOT(adjustColumn()));
     popupMenuActions.insert("adjustColumn", action);
 
-    action = new KAction(KIcon("insert_table_col"), i18n("Insert Columns"), q);
+    action = new KAction(KIcon("edit-table-insert-column-left"), i18n("Insert Columns"), q);
     connect(action, SIGNAL(triggered(bool)), q, SLOT(insertColumn()));
     popupMenuActions.insert("insertColumn", action);
 
-    action = new KAction(KIcon("delete_table_col"), i18n("Delete Columns"), q);
+    action = new KAction(KIcon("edit-table-delete-column"), i18n("Delete Columns"), q);
     connect(action, SIGNAL(triggered(bool)), q, SLOT(deleteColumn()));
     popupMenuActions.insert("deleteColumn", action);
 
@@ -1281,17 +1280,26 @@ void CellToolBase::Private::createPopupMenuActions()
     connect(action, SIGNAL(triggered(bool)), q, SLOT(adjustRow()));
     popupMenuActions.insert("adjustRow", action);
 
-    action = new KAction(KIcon("insert_table_row"), i18n("Insert Rows"), q);
+    action = new KAction(KIcon("edit-table-insert-row-above"), i18n("Insert Rows"), q);
     connect(action, SIGNAL(triggered(bool)), q, SLOT(insertRow()));
     popupMenuActions.insert("insertRow", action);
 
-    action = new KAction(KIcon("delete_table_row"), i18n("Delete Rows"), q);
+    action = new KAction(KIcon("edit-table-delete-row"), i18n("Delete Rows"), q);
     connect(action, SIGNAL(triggered(bool)), q, SLOT(deleteRow()));
     popupMenuActions.insert("deleteRow", action);
 
     action = new KAction(i18n("Selection List..."), q);
     connect(action, SIGNAL(triggered(bool)), q, SLOT(listChoosePopupMenu()));
     popupMenuActions.insert("listChoose", action);
+
+    action = new KAction(KIcon("comment"), i18n("Comment"), q);
+    connect(action, SIGNAL(triggered(bool)), q, SLOT(comment()));
+    popupMenuActions.insert("comment", action);
+
+    action = new KAction(KIcon("clearComment"),i18n("Clear Comment"), q);
+    connect(action, SIGNAL(triggered(bool)), q, SLOT(clearComment()));
+    popupMenuActions.insert("clearComment", action);
+
 }
 
 bool CellToolBase::Private::testListChoose(Selection *selection) const
