@@ -5,7 +5,7 @@
  * Copyright (C) 2008 Pierre Ducroquet <pinaraf@pinaraf.info>
  * Copyright (C) 2008 Sebastian Sauer <mail@dipe.org>
  * Copyright (C) 2010 Boudewijn Rempt <boud@kogmbh.com>
- * Copyright (C) 2010 Casper Boemann <cbo@kogmbh.com>
+ * Copyright (C) 2010 C. Boemann <cbo@kogmbh.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -52,6 +52,7 @@
 #include <KoShapeContainer.h>
 #include <KoOdfWriteStore.h>
 #include <KoToolManager.h>
+#include <KoShapeController.h>
 #include <KoShapeRegistry.h>
 #include <KoShapeFactoryBase.h>
 #include <KoStyleManager.h>
@@ -90,8 +91,8 @@
 #include <QTextBlock>
 #include <QTime>
 
-KWDocument::KWDocument(QWidget *parentWidget, QObject *parent, bool singleViewMode)
-        : KoDocument(parentWidget, parent, singleViewMode),
+KWDocument::KWDocument(QObject *parent)
+        : KoDocument(parent),
         m_frameLayout(&m_pageManager, m_frameSets),
         m_mainFramesetEverFinished(false)
 {
@@ -115,10 +116,12 @@ KWDocument::KWDocument(QWidget *parentWidget, QObject *parent, bool singleViewMo
     if (documentRdfBase()) {
         documentRdfBase()->linkToResourceManager(resourceManager());
     }
-
+/* TODO reenable after release
     QVariant variant;
     variant.setValue(new KoChangeTracker(resourceManager()));
     resourceManager()->setResource(KoText::ChangeTracker, variant);
+*/
+    m_shapeController = new KoShapeController(0, this);
 
     if (inlineTextObjectManager()) {
         connect(documentInfo(), SIGNAL(infoUpdated(const QString &, const QString &)),
@@ -140,6 +143,11 @@ KWDocument::~KWDocument()
 // will call addFrameSet.  Which will itself call addFrame()
 // any call coming in here is due to the undo/redo framework, pasting or for nested frames
 void KWDocument::addShape(KoShape *shape)
+{
+    addShape(shape, 0);
+}
+
+void KWDocument::addShape(KoShape* shape, KoTextAnchor* anchor)
 {
     KWFrame *frame = dynamic_cast<KWFrame*>(shape->applicationData());
     kDebug(32001) << "shape=" << shape << "frame=" << frame;
@@ -147,11 +155,11 @@ void KWDocument::addShape(KoShape *shape)
         if (shape->shapeId() == TextShape_SHAPEID) {
             KWTextFrameSet *tfs = new KWTextFrameSet(this);
             tfs->setName("Text");
-            frame = new KWFrame(shape, tfs);
+            frame = new KWFrame(shape, tfs, anchor);
         } else {
             KWFrameSet *fs = new KWFrameSet();
             fs->setName(shape->shapeId());
-            frame = new KWFrame(shape, fs);
+            frame = new KWFrame(shape, fs, anchor);
         }
     }
     Q_ASSERT(frame->frameSet());
@@ -162,12 +170,13 @@ void KWDocument::addShape(KoShape *shape)
         KoCanvasBase *canvas = static_cast<KWView*>(view)->canvasBase();
         canvas->shapeManager()->addShape(shape);
     }
+    shape->update();
 }
 
 void KWDocument::removeShape(KoShape *shape)
 {
     KWFrame *frame = dynamic_cast<KWFrame*>(shape->applicationData());
-    kDebug(32001) << "shape=" << shape << "frame=" << frame << "frameSetType=" << (frame ? Words::frameSetTypeName(frame->frameSet()) : QString());
+    qDebug() << "shape=" << shape << "frame=" << frame << "frameSetType=" << (frame ? Words::frameSetTypeName(frame->frameSet()) : QString());
     if (frame) { // not all shapes have to have a frame. Only top-level ones do.
         KWFrameSet *fs = frame->frameSet();
         Q_ASSERT(fs);
@@ -183,6 +192,26 @@ void KWDocument::removeShape(KoShape *shape)
     }
 }
 
+void KWDocument::shapesRemoved(const QList<KoShape*> &shapes, KUndo2Command *command)
+{
+    QMap<KoTextEditor *, QList<KoTextAnchor *> > anchors;
+    foreach (KoShape *shape, shapes) {
+        KWFrame *frame = dynamic_cast<KWFrame*>(shape->applicationData());
+        if (frame && frame->shape()) {
+            KoTextAnchor *anchor = frame->anchor();
+            const QTextDocument *document = anchor ? anchor->document(): 0;
+            if (document) {
+                KoTextEditor *editor = KoTextDocument(document).textEditor();
+                anchors[editor].append(anchor);
+            }
+        }
+    }
+    QMap<KoTextEditor *, QList<KoTextAnchor *> >::const_iterator it(anchors.constBegin());
+    for (; it != anchors.constEnd(); ++it) {
+        it.key()->removeAnchors(it.value(), command);
+    }
+}
+
 void KWDocument::paintContent(QPainter &, const QRect &)
 {
 }
@@ -432,6 +461,7 @@ void KWDocument::removeFrame(KWFrame *frame)
 {
     if (frame->shape() == 0) return;
     kDebug(32001) << "frame=" << frame << "frameSet=" << frame->frameSet();
+
     removeFrameFromViews(frame);
     KWPage page = pageManager()->page(frame->shape());
     if (!page.isValid()) return;
@@ -727,14 +757,6 @@ bool KWDocument::saveOdf(SavingContext &documentContext)
     return writer.save(documentContext.odfStore, documentContext.embeddedSaver);
 }
 
-QStringList KWDocument::extraNativeMimeTypes(ImportExportType importExportType) const
-{
-    QStringList answer = KoDocument::extraNativeMimeTypes(importExportType);
-    if (importExportType == KoDocument::ForExport)
-        answer.removeAll("application/x-words"); // we can't save this, only load.
-    return answer;
-}
-
 void KWDocument::updatePagesForStyle(const KWPageStyle &style)
 {
     kDebug(32001) << "pageStyleName=" << style.name();
@@ -806,11 +828,83 @@ void KWDocument::saveConfig()
         return;
 //   KConfigGroup group(KoGlobal::calligraConfig(), "Spelling");
 //   group.writeEntry("PersonalDict", m_spellCheckPersonalDict);
-    if (isEmbedded())
-        return;
+
     m_config.save();
     KSharedConfigPtr config = KGlobal::config();
     KConfigGroup interface = config->group("Interface");
     interface.writeEntry("ResolutionX", gridData().gridX());
     interface.writeEntry("ResolutionY", gridData().gridY());
 }
+
+KWFrame* KWDocument::findClosestFrame(KoShape* shape) const
+{
+    KWFrame *result = 0;
+    int      area   = 0;
+    QRectF   br     = shape->boundingRect();
+
+    // now find the frame that is closest to the frame we want to inline.
+    foreach (KWFrame *frame, mainFrameSet()->frames()) {
+        QRectF intersection  = br.intersected(frame->shape()->boundingRect());
+        int    intersectArea = qRound(intersection.width() * intersection.height());
+
+        if (intersectArea > area) {
+            result = frame;
+            area   = intersectArea;
+        } else if (result == 0) {
+            // TODO check distance between frames or something.
+        }
+    }
+
+    return result;
+}
+
+KoTextAnchor* KWDocument::anchorOfShape(KoShape *shape) const
+{
+    Q_ASSERT(mainFrameSet());
+    Q_ASSERT(shape);
+
+    // try and find out if shape is already anchored    
+    foreach (KoInlineObject *inlineObject, inlineTextObjectManager()->inlineTextObjects()) {
+        KoTextAnchor *anchor = dynamic_cast<KoTextAnchor *>(inlineObject);
+        if (anchor && anchor->shape() == shape) {
+            return anchor;
+        }
+    }
+
+    KWFrame *frame = frameOfShape(shape);
+    KoTextAnchor *anchor = frame->anchor();
+
+    if (!anchor) {
+        anchor = new KoTextAnchor(shape);
+        anchor->setAnchorType(KoTextAnchor::AnchorPage);
+        anchor->setHorizontalPos(KoTextAnchor::HFromLeft);
+        anchor->setVerticalPos(KoTextAnchor::VFromTop);
+        frame->setAnchor(anchor);
+    }
+
+    return anchor;
+}
+
+
+KWFrame *KWDocument::frameOfShape(KoShape* shape) const
+{
+    while (shape) {
+        KWFrame *answer = dynamic_cast<KWFrame*>(shape->applicationData());
+        if (answer)
+            return answer;
+        if (shape->parent() == 0)
+            break;
+        shape = shape->parent();
+    }
+
+    KWFrame *answer = dynamic_cast<KWFrame*>(shape->applicationData());
+    if (answer == 0) { // this may be a clipping shape containing the frame-shape
+        KoShapeContainer *container = dynamic_cast<KoShapeContainer*>(shape);
+        if (container && container->shapeCount() == 1) {
+            answer = dynamic_cast<KWFrame*>(container->shapes()[0]->applicationData());
+        }
+    }
+
+    return answer;
+}
+
