@@ -40,6 +40,7 @@ class KWGui;
 class KoCanvasBase;
 class KoZoomController;
 class KoFindText;
+class KoFindStyle;
 class KoRdfSemanticItem;
 class KoTextAnchor;
 
@@ -138,6 +139,8 @@ private:
     KoShape *selectedShape() const;
 
 private slots:
+    /// create a template from document
+    void createTemplate();
     /// displays the KWFrameDialog that allows to alter the frameset properties
     void editFrameProperties();
     /// called if another shape got selected
@@ -168,14 +171,12 @@ private slots:
     void editSelectAllFrames();
     /// calls delete on the active tool
     void editDeleteSelection();
-    /// Wrap the selected frames into a clipping shape container.
-    void createFrameClipping();
-    /// unwrap the selected frames into a clipping shape container.
-    void removeFrameClipping();
     /** decide if we enable or disable the action "delete_page" uppon m_document->page_count() */
     void updateStatusBarAction();
     /// show guides menu option uses this
     void setGuideVisibility(bool on);
+    /// open the configure dialog.
+    void configure();
     /// A semantic item was updated and should have it's text refreshed.
     void semanticObjectViewSiteUpdated(KoRdfSemanticItem *item, const QString &xmlid);
     /// A match was found when searching.
@@ -196,6 +197,7 @@ private:
     KWPage m_currentPage;
     KoFindText *m_find;
 
+    KAction *m_actionCreateTemplate;
     KAction *m_actionFormatFrameSet;
     KAction *m_actionInsertFrameBreak;
     KAction *m_actionAddBookmark;
