@@ -132,7 +132,6 @@ void KWFrameLayout::createNewFramesForPage(int pageNumber)
             shape->setPosition(QPointF(0, page.offsetInDocument()));
             background = new KWFrame(shape, m_backgroundFrameSet);
             shape->setTextRunAroundSide(KoShape::RunThrough);
-            // TODO pagespread
         }
         background->shape()->setBackground(page.pageStyle().background());
     } else {
@@ -163,10 +162,7 @@ void KWFrameLayout::createNewFramesForPage(int pageNumber)
     Q_ASSERT(columns >= 1);
     KWTextFrameSet *fs = getOrCreate(Words::MainTextFrameSet, page);
     QRectF rect(QPointF(0, page.offsetInDocument()), QSizeF(page.width(), page.height()));
-#if 0
-        if (page.pageSide() == KWPage::PageSpread)
-            rect.setWidth(rect.width() / 2);
-#endif
+
     kDebug(32001) << "MainTextFrame" << fs << "pageRect=" << rect << "columns=" << columns;
     foreach (KWFrame *frame, framesInPage(rect)) {
         if (frame->frameSet() == fs) {
@@ -189,116 +185,7 @@ void KWFrameLayout::createNewFramesForPage(int pageNumber)
         new KWFrame(shape, fs);
     }
 
-#if 0
-    if (page.pageSide() == KWPage::PageSpread) {
-
-        // inline helper class
-        class PageSpreadShapeFactory
-        {
-        public:
-            PageSpreadShapeFactory(KWFrameLayout *parent) {
-                m_parent = parent;
-            }
-            void create(const KWPage &page, KWTextFrameSet *fs) {
-                KWFrame *frame;
-                if (fs->textFrameSetType() == Words::MainTextFrameSet) {
-                    KoShape * shape = m_parent->createTextShape(page);
-                    shape->setPosition(QPointF(page.width() / 2 + 1, shape->position().y()));
-                    shape->setSize(QSizeF(page.pageStyle().pageLayout().width, page.pageStyle().pageLayout().height));
-                    frame = new KWTextFrame(shape, fs);
-                } else {
-                    frame = m_parent->createCopyFrame(fs, page);
-                    frame->shape()->setPosition(QPointF(page.width() / 2 + 1, frame->shape()->position().y()));
-                    frame->shape()->setSize(QSizeF(page.pageStyle().pageLayout().width, page.pageStyle().pageLayout().height));
-                }
-            }
-            KWFrameLayout *m_parent;
-        };
-        PageSpreadShapeFactory factory(this);
-#if 0
-        if (shouldHaveHeaderOrFooter(pageNumber + 1, true, &origin)) {
-            KWTextFrameSet *fs = getOrCreate(origin, m_pageManager->page(pageNumber + 1));
-            if (!frameOn(fs, pageNumber + 1))
-                factory.create(page, fs);
-        }
-        if (shouldHaveHeaderOrFooter(pageNumber + 1, false, &origin)) {
-            KWTextFrameSet *fs = getOrCreate(origin, m_pageManager->page(pageNumber + 1));
-            if (!frameOn(fs, pageNumber + 1))
-                factory.create(page, fs);
-        }
-#else
-    #ifdef __GNUC__
-        #warning FIXME: port to textlayout-rework
-    #endif
-#endif
-        if (page.pageStyle().hasMainTextFrame()) {
-            int columns = page.pageStyle().columns().columns;
-            KWTextFrameSet *fs = getOrCreate(Words::MainTextFrameSet, page);
-            QRectF rect(QPointF(page.width(), page.offsetInDocument()),
-                        QSizeF(page.width() / 2,  page.height()));
-#if 0
-            foreach (KWFrame *frame, framesInPage(rect)) {
-                if (frame->frameSet() == fs) {
-                    columns--;
-                    if (columns < 0) {
-                        fs->removeFrame(frame);
-                        delete frame;
-                    }
-                }
-            }
-#endif
-            while (columns > 0) {
-                factory.create(page, fs);
-                columns--;
-            }
-        }
-    }
-#endif
-
-#if 0
-    bool odd = false; // an odd number of pages back, so frameOnBothSheets matters
-    for (int i = pageNumber - 2; i < pageNumber; i++) {
-        if (i < m_pageManager->begin().pageNumber()) {
-            odd = true;
-            continue;
-        }
-
-        KWPage prevPage = m_pageManager->page(i);
-        QRectF pageRect = prevPage.rect(pageNumber);
-        foreach (KWFrame *frame, framesInPage(pageRect)) {
-            if (odd && !frame->frameOnBothSheets())
-                continue;
-            if (!(frame->newFrameBehavior() == Words::ReconnectNewFrame ||
-                    frame->newFrameBehavior() == Words::CopyNewFrame))
-                continue;
-            if (Words::isAutoGenerated(frame->frameSet()))
-                continue; // these are copied above already.
-            KWFrame *f = 0;
-            KWTextFrameSet *tfs = dynamic_cast<KWTextFrameSet*>(frame->frameSet());
-            if (tfs && frame->newFrameBehavior() != Words::CopyNewFrame) {
-                f = new KWTextFrame(createTextShape(page), tfs);
-            } else {
-                Q_ASSERT(frame->newFrameBehavior() == Words::CopyNewFrame);
-                Q_ASSERT(frame->frameSet());
-                Q_ASSERT(frame->frameSet()->frameCount() > 0);
-                f = new KWFrame(new KWCopyShape(frame->frameSet()->frames().first()->shape(),
-                            m_pageManager), frame->frameSet());
-            }
-            const qreal y = frame->shape()->position().y();
-            qreal offsetFromPage = y - pageRect.top();
-            f->copySettings(frame);
-            f->shape()->setPosition(QPointF(frame->shape()->position().x(),
-                                            page.offsetInDocument() + offsetFromPage));
-        }
-        odd = true;
-    }
-#endif
-
     layoutFramesOnPage(pageNumber);
-#if 0
-    if (page.pageSide() == KWPage::PageSpread)
-        layoutFramesOnPage(pageNumber + 1);
-#endif
 }
 
 void KWFrameLayout::layoutFramesOnPage(int pageNumber)
@@ -317,23 +204,21 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
        |  1  [ header ]  |
        |  2              | <- pageStyle->headerDistance()
        |  3  [ maintxt ] |
-       |  4              | <- pageStyle->endNoteDistance()
-       |  5  [ endnote ] |
-       |  6              | <- pageStyle->footerDistance()
-       |  7  [ footer ]  |
-       |  8              | <- pageStyle->pageLayout()->bottomMargin + layout->bottomPadding
+       |  4              | <- pageStyle->footerDistance()
+       |  5  [ footer ]  |
+       |  6              | <- pageStyle->pageLayout()->bottomMargin + layout->bottomPadding
        +-----------------+ */
 
     // Create some data structures used for the layouting of the frames later
     int minZIndex = INT_MAX;
-    qreal requestedHeight[9], minimumHeight[9], resultingPositions[9];
-    for (int i = 0; i < 9; i++) { // zero fill.
+    qreal requestedHeight[7], minimumHeight[7], resultingPositions[7];
+    for (int i = 0; i < 7; i++) { // zero fill.
         requestedHeight[i] = 0;
         minimumHeight[i] = 0;
         resultingPositions[i] = 0;
     }
     minimumHeight[0] = page.topMargin() + page.topPadding();
-    minimumHeight[8] = page.bottomMargin() + page.bottomPadding();
+    minimumHeight[6] = page.bottomMargin() + page.bottomPadding();
 
     KoPageLayout layout = page.pageStyle().pageLayout();
     layout.leftMargin = page.leftMargin();
@@ -341,16 +226,6 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
     layout.leftPadding = page.leftPadding();
     layout.rightPadding = page.rightPadding();
     qreal left = 0, width = page.width();
-    if (page.pageSide() == KWPage::PageSpread) {
-        width /= 2;
-        layout.leftMargin = page.pageEdgeMargin();
-        layout.rightMargin = page.marginClosestBinding();
-        if (page.pageNumber() != pageNumber) { // doing the 'right' part
-            left = width;
-            qSwap(layout.leftMargin, layout.rightMargin);
-            qSwap(layout.leftPadding, layout.rightPadding);
-        }
-    }
     qreal textWidth = width - layout.leftMargin - layout.rightMargin
                             - layout.leftPadding - layout.rightPadding;
 
@@ -359,7 +234,7 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
     int columnsCount = columns;
     int columnIndex = 0;
     KWFrame **main;
-    KWFrame *footer = 0, *endnote = 0, *header = 0;
+    KWFrame *footer = 0, *header = 0;
     KWFrame *pageBackground = 0;
     main = new KWFrame*[columnsCount];
     if (columns > 0)
@@ -403,17 +278,23 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
         case Words::OddPagesHeaderTextFrameSet:
         case Words::EvenPagesHeaderTextFrameSet: {
             header = frame;
-            minimumHeight[1] = qMax((qreal)10, pageStyle.headerMinimumHeight());
             minimumHeight[2] = pageStyle.headerDistance();
-            requestedHeight[1] = qMax(pageStyle.headerMinimumHeight(), textFrameSet->frames().first()->minimumFrameHeight());
+            minimumHeight[1] = qMax((qreal)10, pageStyle.headerMinimumHeight() - pageStyle.headerDistance());
+            requestedHeight[1] = qMax(minimumHeight[1], textFrameSet->frames().first()->minimumFrameHeight());
+            if (pageStyle.headerDynamicSpacing()) {
+                minimumHeight[2] = qMax((qreal)0, minimumHeight[1] - requestedHeight[1]);
+            }
             break;
         }
         case Words::OddPagesFooterTextFrameSet:
         case Words::EvenPagesFooterTextFrameSet: {
             footer = frame;
-            minimumHeight[7] = qMax((qreal)10, pageStyle.footerMinimumHeight());
-            minimumHeight[6] = pageStyle.footerDistance();
-            requestedHeight[7] = qMax(pageStyle.footerMinimumHeight(), textFrameSet->frames().first()->minimumFrameHeight());
+            minimumHeight[4] = pageStyle.footerDistance();
+            minimumHeight[5] = qMax((qreal)10, pageStyle.footerMinimumHeight() - pageStyle.footerDistance());
+            requestedHeight[5] = qMax(minimumHeight[5], textFrameSet->frames().first()->minimumFrameHeight());
+            if (pageStyle.headerDynamicSpacing()) {
+                minimumHeight[4] = qMax((qreal)0, minimumHeight[5] - requestedHeight[5]);
+            }
             break;
         }
         case Words::MainTextFrameSet: {
@@ -439,9 +320,6 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
     pageBackground = frameOn(m_backgroundFrameSet, pageNumber);
 
     --minZIndex;
-    if (endnote) {
-        endnote->shape()->setZIndex(minZIndex--);
-    }
     for (int i = 0; i < columns; ++i) {
         Q_ASSERT_X(main[i], __FUNCTION__, QString("No KWFrame for column=%1 columnCount=%2").arg(i).arg(columns).toLocal8Bit());
         Q_ASSERT_X(main[i]->shape(), __FUNCTION__, QString("No TextShape in KWFrame for column=%1 columnCount=%2").arg(i).arg(columns).toLocal8Bit());
@@ -449,14 +327,19 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
             main[i]->shape()->setZIndex(minZIndex);
     }
     if (footer && footer->shape()) {
-        footer->shape()->setZIndex(--minZIndex);
+        footer->shape()->setZIndex(minZIndex);
+        // Make us compatible with ms word (seems saner too). Compatible with LO would be 0
+        footer->shape()->setRunThrough(-3); //so children will be <= -2 and thus below main text
     }
     if (header && header->shape()) {
-        header->shape()->setZIndex(--minZIndex);
+        header->shape()->setZIndex(minZIndex);
+        // Make us compatible with ms word (seems saner too). Compatible with LO would be 0
+        header->shape()->setRunThrough(-3); //so children will be <= -2 and thus below main text
     }
 
     if (pageBackground) {
         KoShape *bs = pageBackground->shape();
+        bs->setRunThrough(-10); //so it will be below everything
         bs->setZIndex(--minZIndex);
         bs->setSize(pageRect.size());
         bs->setPosition(pageRect.topLeft());
@@ -464,27 +347,23 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
 
     // spread space across items.
     qreal heightLeft = page.height();
-    for (int i = 0; i < 9; i++)
+    for (int i = 0; i < 7; i++)
         heightLeft -= qMax(minimumHeight[i], requestedHeight[i]);
     if (heightLeft >= 0) { // easy; plenty of space
-        if (minimumHeight[5] > 0) // if we have an endnote
-            minimumHeight[6] += heightLeft; // add space below endnote
-        else
-            minimumHeight[3] += heightLeft; // add space to main text frame
+        minimumHeight[3] += heightLeft; // add space to main text frame
         qreal y = page.offsetInDocument();
-        for (int i = 0; i < 9; i++) {
+        for (int i = 0; i < 7; i++) {
             resultingPositions[i] = y;
             y += qMax(minimumHeight[i], requestedHeight[i]);
         }
     } else {
         // for situations where the header + footer are too big to fit together with a
         // minimum sized main text frame.
-        minimumHeight[5] = 0; // no end note
         heightLeft = page.height();
-        for (int i = 0; i < 9; i++)
+        for (int i = 0; i < 7; i++)
             heightLeft -= minimumHeight[i];
         qreal y = page.offsetInDocument();
-        for (int i = 0; i < 9; i++) {
+        for (int i = 0; i < 7; i++) {
             resultingPositions[i] = y;
             qreal row = minimumHeight[i];
             if (requestedHeight[i] > row) {
@@ -496,11 +375,12 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
 
     // actually move / size the frames.
     if (columns > 0 && main[0]) {
-        const qreal columnWidth = textWidth / columns;
+        const qreal columnWidth = (textWidth
+                - page.pageStyle().columns().columnSpacing * (columns- 1 ))/ columns;
+        const qreal columnStep = columnWidth + page.pageStyle().columns().columnSpacing;
         QPointF *points = new QPointF[columns];
-        for (int i = columns - 1; i >= 0; i--)
-            points[i] = QPointF(left + layout.leftMargin + layout.leftPadding
-                                + columnWidth * i, resultingPositions[3]);
+        for (int i = 0; i < columns; i++)
+            points[i] = QPointF(left + layout.leftMargin + layout.leftPadding +columnStep * i, resultingPositions[3]);
         for (int i = 0; i < columns; i++) {
             for (int f = 0; f < columns; f++) {
                 if (f == i) continue;
@@ -511,23 +391,18 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
             }
         }
 
-        bool first = true;
         for (int i = columns - 1; i >= 0; i--) {
             main[i]->setFrameBehavior(Words::AutoCreateNewFrameBehavior);
             main[i]->setNewFrameBehavior(Words::ReconnectNewFrame);
             KoShape *shape = main[i]->shape();
             shape->setPosition(points[i]);
-            shape->setSize(QSizeF(columnWidth -
-                                  (first ? 0 : page.pageStyle().columns().columnSpacing),
-                                  resultingPositions[4] - resultingPositions[3]));
-            first = false;
+            shape->setSize(QSizeF(columnWidth, resultingPositions[4] - resultingPositions[3]));
         }
         delete[] points;
-    }
-    if (endnote) {
-        endnote->shape()->setPosition(
-            QPointF(left + layout.leftMargin + layout.leftPadding, resultingPositions[5]));
-        endnote->shape()->setSize(QSizeF(textWidth, resultingPositions[6] - resultingPositions[5]));
+
+        // We need to store the content rect so layout can place it's anchored shapes
+        // correctly
+        page.setContentRect(QRectF(QPointF(left + layout.leftMargin + layout.leftPadding, resultingPositions[3]), QSizeF(textWidth ,resultingPositions[4] - resultingPositions[3])));
     }
     if (header) {
         header->shape()->setPosition(
@@ -536,8 +411,8 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
     }
     if (footer) {
         footer->shape()->setPosition(
-            QPointF(left + layout.leftMargin + layout.leftPadding, resultingPositions[7]));
-        footer->shape()->setSize(QSizeF(textWidth, resultingPositions[8] - resultingPositions[7]));
+            QPointF(left + layout.leftMargin + layout.leftPadding, resultingPositions[5]));
+        footer->shape()->setSize(QSizeF(textWidth, resultingPositions[6] - resultingPositions[5]));
     }
     if (pageBackground) {
         pageBackground->shape()->setPosition(
@@ -571,10 +446,10 @@ bool KWFrameLayout::shouldHaveHeaderOrFooter(int pageNumber, bool header, Words:
             }
         } break;
         case KWPageStyle::LeftPages:
-            *origin = Words::EvenPagesHeaderTextFrameSet;
+            *origin = header ? Words::EvenPagesHeaderTextFrameSet : Words::EvenPagesFooterTextFrameSet;
             break;
         case KWPageStyle::RightPages:
-            *origin = Words::OddPagesHeaderTextFrameSet;
+            *origin = header ? Words::OddPagesHeaderTextFrameSet : Words::OddPagesFooterTextFrameSet;
             break;
     }
     return true;
